import unittest
import datetime
import unicodedata

from imap_tools.errors import ImapToolsError, UnexpectedCommandStatusError, MailboxCopyError
from imap_tools.consts import MailMessageFlags
from imap_tools.utils import clean_flags, chunked, quote, pairs_to_dict, decode_value, check_command_status, \
    parse_email_date, parse_email_addresses, EmailAddress, clean_uids, replace_html_ct_charset, chunked_crop, \
    remove_non_printable


class UtilsTest(unittest.TestCase):

    def test_remove_non_printable(self):
        all_non_printable_chars = []
        for i in range(0x110000):  # Диапазон всех кодовых точек Unicode
            char = chr(i)
            if unicodedata.category(char).startswith('C'):
                all_non_printable_chars.append(char)
        self.assertEqual(remove_non_printable('123{}'.format(''.join(all_non_printable_chars))), '123')

    def test_clean_uids(self):
        # *clean_uids also implicitly tested in test_query.py
        self.assertEqual(clean_uids('11'), ['11'])
        self.assertEqual(clean_uids('1,2'), ['1', '2'])
        self.assertEqual(clean_uids(' 1,2, 4 '), ['1', '2', '4'])
        self.assertEqual(clean_uids('1,222'), ['1', '222'])
        self.assertEqual(clean_uids('1,2:*'), ['1', '2:*'])
        for i in ['', 1, 1.0, dict(a=1)]:
            with self.assertRaises(TypeError):
                clean_uids(i)  # noqa

    def test_clean_flags(self):
        self.assertEqual(clean_flags([MailMessageFlags.FLAGGED, MailMessageFlags.SEEN]), ['\\Flagged', '\\Seen'])
        self.assertEqual(clean_flags(['\\FLAGGED', '\\seen']), ['\\FLAGGED', '\\seen'])
        self.assertEqual(clean_flags(['TAG1']), ['TAG1'])
        self.assertEqual(clean_flags(['tag2']), ['tag2'])
        for flag in MailMessageFlags.all:
            self.assertEqual(clean_flags(flag), ['\\' + flag.replace('\\', '', 1).capitalize()])
        with self.assertRaises(ValueError):
            clean_flags([MailMessageFlags.FLAGGED, '\\CUSTOM_TAG_WITH_SLASH'])

    def test_chunked(self):
        self.assertEqual(list(chunked('ABCDE', 2, '=')), [('A', 'B'), ('C', 'D'), ('E', '=')])
        self.assertEqual(list(chunked([1, 2, 3, 4, 5, 6], 3)), [(1, 2, 3), (4, 5, 6)])
        self.assertEqual(list(chunked([], 4)), [])
        self.assertEqual(list(chunked([1, 2], 0)), [])
        self.assertEqual(list(chunked(['0', '0'], 1)), [('0',), ('0',)])

    def test_chunked_crop(self):
        self.assertEqual(list(chunked_crop([1, 2, 3, 4, 5, 6, 7], 3)), [[1, 2, 3], [4, 5, 6], [7]])
        self.assertEqual(list(chunked_crop([1, 2, 3, 4, 5, 6], 3)), [[1, 2, 3], [4, 5, 6]])
        self.assertEqual(list(chunked_crop([1, ], 3)), [[1]])
        self.assertEqual(list(chunked_crop([1], 0)), [[1]])
        self.assertEqual(list(chunked_crop([1, 2], False)), [[1, 2]])
        self.assertEqual(list(chunked_crop([1, 2, 3], None)), [[1, 2, 3]])
        with self.assertRaises(ValueError):
            list(chunked_crop([1], -1))

    def test_quote(self):
        self.assertEqual(quote('str привет'), '"str привет"')
        self.assertEqual(quote('str \\'), '"str \\\\"')
        self.assertEqual(quote(b'\xd1\x8f'), b'"\xd1\x8f"')
        self.assertEqual(quote(b'\\ \xd1\x8f  \\'), b'"\\\\ \xd1\x8f  \\\\"')

    def test_pairs_to_dict(self):
        self.assertEqual(pairs_to_dict(['MESSAGES', '3', 'UIDNEXT', '4']), {'MESSAGES': '3', 'UIDNEXT': '4'})
        with self.assertRaises(ValueError):
            pairs_to_dict(['1', '2', '3'])

    def test_decode_value(self):
        self.assertEqual(decode_value('str привет 你好', 'not matter'), 'str привет 你好')
        self.assertEqual(decode_value(b'str \xd0\xb4\xd0\xb0 \xe4\xbd\xa0'), 'str да 你')
        self.assertEqual(decode_value(b'str \xd0\xb4\xd0\xb0 \xe4\xbd\xa0', 'utf8'), 'str да 你')
        self.assertEqual(decode_value(b'\xef\xf0\xe8\xe2\xe5\xf2', 'cp1251'), 'привет')
        self.assertEqual(decode_value(b'str \xd0\xb4\xd0\xb0 \xe4\xbd\xa0', 'wat?'), 'str да 你')

    def test_check_command_status(self):
        self.assertIsNone(check_command_status(('EXP', 'command_result_data'), MailboxCopyError, expected='EXP'))
        self.assertIsNone(check_command_status(('OK', 'res'), UnexpectedCommandStatusError))
        with self.assertRaises(TypeError):
            check_command_status(('NOT_OK', 'test'), ImapToolsError)
        with self.assertRaises(MailboxCopyError):
            check_command_status(('BYE', ''), MailboxCopyError, expected='OK')

    def test_parse_email_date(self):
        for val, exp in (
                ('1 Jan 2000 00:00', datetime.datetime(2000, 1, 1, 0, 0)),
                ('1 Feb 2000 00:00', datetime.datetime(2000, 2, 1, 0, 0)),
                ('1 Mar 2000 00:00', datetime.datetime(2000, 3, 1, 0, 0)),
                ('1 Apr 2000 00:00', datetime.datetime(2000, 4, 1, 0, 0)),
                ('1 May 2000 00:00', datetime.datetime(2000, 5, 1, 0, 0)),
                ('1 Jun 2000 00:00', datetime.datetime(2000, 6, 1, 0, 0)),
                ('1 Jul 2000 00:00', datetime.datetime(2000, 7, 1, 0, 0)),
                ('1 Aug 2000 00:00', datetime.datetime(2000, 8, 1, 0, 0)),
                ('1 Sep 2000 00:00', datetime.datetime(2000, 9, 1, 0, 0)),
                ('1 Oct 2000 00:00', datetime.datetime(2000, 10, 1, 0, 0)),
                ('1 Nov 2000 00:00', datetime.datetime(2000, 11, 1, 0, 0)),
                ('1 Dec 2000 00:00', datetime.datetime(2000, 12, 1, 0, 0)),

                ('=) wat 7 Jun 2017 09:23!',
                 datetime.datetime(2017, 6, 7, 9, 23)),
                ('7 Jun 2017 09:23',
                 datetime.datetime(2017, 6, 7, 9, 23)),
                ('Wed, 7 Jun 2017 09:23',
                 datetime.datetime(2017, 6, 7, 9, 23)),
                ('Wed, 7 Jun 2017 09:23:14',
                 datetime.datetime(2017, 6, 7, 9, 23, 14)),
                ('Wed, 7 Jun 2017 09:23:14 +0000',
                 datetime.datetime(2017, 6, 7, 9, 23, 14, tzinfo=datetime.timezone.utc)),
                ('Wed, 7 Jun 2017 09:23:14 +0000 (UTC)',
                 datetime.datetime(2017, 6, 7, 9, 23, 14, tzinfo=datetime.timezone.utc)),
                ('Wed, 7 Jun 2017 09:23 +0000',
                 datetime.datetime(2017, 6, 7, 9, 23, tzinfo=datetime.timezone.utc)),
                ('Wed, 7 Jun 2017 09:23 +0000 (UTC)',
                 datetime.datetime(2017, 6, 7, 9, 23, tzinfo=datetime.timezone.utc)),
                ('7 Jun 2017 09:23 +0000',
                 datetime.datetime(2017, 6, 7, 9, 23, tzinfo=datetime.timezone.utc)),
                ('7 Jun 2017 09:23 +0000 (UTC)',
                 datetime.datetime(2017, 6, 7, 9, 23, tzinfo=datetime.timezone.utc)),
                ('7 Jun 2017 09:23 -2359',
                 datetime.datetime(2017, 6, 7, 9, 23, tzinfo=datetime.timezone(datetime.timedelta(-1, 60)))),
                ('7 Jun 2017 09:23 +0530 (UTC) asd',
                 datetime.datetime(2017, 6, 7, 9, 23, tzinfo=datetime.timezone(datetime.timedelta(0, 19800)))),

                ('7 Bad 2017 09:23', datetime.datetime(1900, 1, 1, 0, 0)),
        ):
            self.assertEqual(parse_email_date(val), exp)

    def test_parse_email_addresses(self):
        self.assertEqual(
            parse_email_addresses('=?UTF-8?B?0J7Qu9C1=?= <name@company.ru>,\r\n "\'\\"z, z\\"\'" <ya@ya.ru>\f'),
            (EmailAddress('Оле', 'name@company.ru'),
             EmailAddress('\'"z, z"\'', 'ya@ya.ru'))
        )
        self.assertEqual(
            parse_email_addresses(' <ivan@mail.ru>'),
            (EmailAddress('', 'ivan@mail.ru'),)
        )
        self.assertEqual(
            parse_email_addresses('ivan'),
            (EmailAddress('ivan', ''),)
        )
        self.assertEqual(
            parse_email_addresses('你好 <chan@mail.ru>'),
            (EmailAddress('你好', 'chan@mail.ru'),)
        )
        self.assertEqual(
            parse_email_addresses(' "hi" <bad_mail.wow> '),
            (EmailAddress('hi', ''),)
        )
        self.assertEqual(
            parse_email_addresses('=?utf-8?Q?ATO.RU?= <subscriptions@ato.ru>'),
            (EmailAddress('ATO.RU', 'subscriptions@ato.ru'),)
        )

    def test_replace_html_ct_charset(self):
        target = 'charset=utf-8'

        data1 = """
          <head>
            <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">
            <meta http-equiv="content-type" content="text/html; charset=windows-1251">
            <meta name="author" content="cat charset=windows-123">
          </head>
        """
        res1 = replace_html_ct_charset(data1, 'utf-8')
        self.assertIn(target, res1)
        self.assertTrue(res1.count(target) == 1)

        data2 = """
          <head>
            <meta name="viewport" content="width=device-width, initial-scale=1, shrink-to-fit=no">
            < 
            META 
             http-EQUIV="
             Content-TYPE " content="text/html; charset 
             = 
             WINDOWS-1251+ ;"/>
            <meta name="author" content="cat charset=windows-123">
          </head>
        """
        res2 = replace_html_ct_charset(data2, 'utf-8')
        self.assertIn(target, res2)
        self.assertTrue(res2.count(target) == 1)



import sys
import socket
import select
import imaplib
from typing import Optional, List

from .utils import check_command_status
from .errors import MailboxTaggedResponseError

imaplib.Commands.setdefault("IDLE", ("NONAUTH", "AUTH", "SELECTED"))  # noqa

SUPPORTS_SELECT_POLL = hasattr(select, 'poll')


def get_socket_poller(sock: socket.socket, timeout: Optional[int] = None):
    """
    Polls the socket for events telling us it's available to read
    :param sock: socket.socket
    :param timeout: seconds or None - seconds
    :return: polling object
    """
    if SUPPORTS_SELECT_POLL:
        # select.poll allows your process to have more than 1024 file descriptors
        poller = select.poll()
        poller.register(sock.fileno(), select.POLLIN)
        timeout = None if timeout is None else timeout * 1000
        return poller.poll(timeout)
    else:
        # select.select fails if your process has more than 1024 file descriptors, needs for windows and some other
        return select.select([sock], [], [], timeout)[0]


class IdleManager:
    """
    Mailbox IDLE logic
    Info about IMAP4 IDLE command at rfc2177
    Workflow examples:
    1.
        mailbox.idle.start()
        resps = mailbox.idle.poll(timeout=60)
        mailbox.idle.stop()
    2.
        resps = mailbox.idle.wait(timeout=60)
    """

    def __init__(self, mailbox):
        self.mailbox = mailbox
        self._idle_tag = None

    def start(self):
        """Switch on mailbox IDLE mode"""
        self._idle_tag = self.mailbox.client._command('IDLE')  # example: b'KLIG3'
        result = self.mailbox.client._get_response()
        check_command_status((result, 'IDLE start'), MailboxTaggedResponseError, expected=None)
        return result

    def stop(self):
        """Switch off mailbox IDLE mode"""
        self.mailbox.client.send(b"DONE\r\n")
        return self.mailbox.consume_until_tagged_response(self._idle_tag)

    def poll(self, timeout: Optional[float]) -> List[bytes]:
        """
        Poll for IDLE responses
        timeout = None
            Blocks until an IDLE response is received
        timeout = float
            Blocks until IDLE response is received or the timeout will expire
        :param timeout: seconds or None
        :return: list of raw responses
        result examples:
            [b'* 36 EXISTS', b'* 1 RECENT']
            [b'* 7 EXISTS']

        socket.settimeout modes:
            0 - non-blocking
            int - timeout mode
            None - blocking
        """
        if timeout is not None:
            timeout = float(timeout)
            if timeout > 29 * 60:
                raise ValueError(
                    'rfc2177 are advised to terminate the IDLE '
                    'and re-issue it at least every 29 minutes to avoid being logged off.'
                )
        sock = self.mailbox.client.sock
        old_timeout = sock.gettimeout()
        # make socket non-blocking so the timeout can be implemented for this call
        sock.settimeout(0)
        try:
            response_set = []
            events = get_socket_poller(sock, timeout)
            if events:
                while True:
                    try:
                        line = self.mailbox.client._get_line()
                    except (socket.timeout, socket.error):
                        break
                    except imaplib.IMAP4.abort:  # noqa
                        etype, evalue, etraceback = sys.exc_info()
                        if "EOF" in evalue.args[0]:
                            break
                        else:
                            raise
                    else:
                        response_set.append(line)
            return response_set
        finally:
            sock.settimeout(old_timeout)

    def wait(self, timeout: Optional[float]) -> List[bytes]:
        """
        Logic, step by step:
        1. Start idle mode
        2. Poll idle response
        3. Stop idle mode on response or timeout
        4. Return poll results
        :param timeout: for poll method
        :return: poll response
        """
        with self as idle:
            return idle.poll(timeout=timeout)

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, exc_type, exc_value, exc_traceback):
        self.stop()



import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='Testing outlook',
    from_='email_test@me.nowhere',
    to=('mikel@me.nowhere',),
    cc=(),
    bcc=(),
    reply_to=('email_test@me.nowhere',),
    date=datetime.datetime(2007, 10, 21, 19, 38, 13, tzinfo=datetime.timezone(datetime.timedelta(seconds=36000))),
    date_str='Sun, 21 Oct 2007 19:38:13 +1000',
    text='',
    html='',
    headers={'return-path': ('<email_test@me.nowhere>',), 'received': ('from omta05sl.mx.bigpond.com by me.nowhere.else with ESMTP id 632BD5758 for <mikel@me.nowhere.else>; Sun, 21 Oct 2007 19:38:21 +1000', 'from oaamta05sl.mx.bigpond.com by omta05sl.mx.bigpond.com with ESMTP id <20071021093820.HSPC16667.omta05sl.mx.bigpond.com@oaamta05sl.mx.bigpond.com> for <mikel@me.nowhere.else>; Sun, 21 Oct 2007 19:38:20 +1000', 'from mikel091a by oaamta05sl.mx.bigpond.com with SMTP id <20071021093820.JFMT24025.oaamta05sl.mx.bigpond.com@mikel091a> for <mikel@me.nowhere.else>; Sun, 21 Oct 2007 19:38:20 +1000'), 'date': ('Sun, 21 Oct 2007 19:38:13 +1000',), 'from': ('Mikel Lindsaar <email_test@me.nowhere>',), 'reply-to': ('Mikel Lindsaar <email_test@me.nowhere>',), 'to': ('mikel@me.nowhere',), 'message-id': ('<009601c813c6$19df3510$0437d30a@mikel091a>',), 'subject': ('Testing outlook', 'Another PDF'), 'mime-version': ('1.0',), 'content-type': ('multipart/alternative;\r\n  boundary=----=_Part_13069834_15179892.1376435426074',)},
    attachments=[
        dict(
            filename='2013-08-13_19-08-28-1.jpg',
            content_id='',
            content_disposition='',
            content_type='image/jpeg',
            payload=b'BINARY_CONTENT_GOES_HERE',
        ),
        ],
    from_values=EmailAddress(name='Mikel Lindsaar', email='email_test@me.nowhere'),
    to_values=(EmailAddress(name='', email='mikel@me.nowhere'),),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(EmailAddress(name='Mikel Lindsaar', email='email_test@me.nowhere'),),
)


import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='NOTE: 한국말로 하는 것',
    from_='mikel@me.com',
    to=('bob@bob.com',),
    cc=(),
    bcc=(),
    reply_to=(),
    date=datetime.datetime(2008, 9, 20, 20, 4, 30, tzinfo=datetime.timezone(datetime.timedelta(seconds=10800))),
    date_str='Sat, 20 Sep 2008 20:04:30 +0300 (�������� ������ ��������������)',
    text='대부분의 마찬가지로, 우리는 하나님을 믿습니다.\r\n\r\n제 이름은 Jamis입니다.',
    html='',
    headers={'mime-version': ('1.0 (Apple Message framework v622)',), 'received': ('from jsj1wlrmd001.webex.com (by jsj1wlrmd001.webex.com\r\n  (8.12.10/8.12.11) with ESMTP id m8MKKPTs022429\r\n  for <xxxx@example.com>; Mon, 22 Sep 2008 20:20:25 GMT',), 'content-transfer-encoding': ('base64',), 'message-id': ('<d3b8cf8e49f04480850c28713a1f473e@lindsaar.net>',), 'content-type': ('text/plain;\r\n  charset=EUC-KR;\r\n  format=flowed',), 'to': ('bob@bob.com',), 'from': ('mikel@me.com',), 'subject': ('=?EUC-KR?Q?NOTE:_=C7=D1=B1=B9=B8=BB=B7=CE_=C7=CF=B4=C2_=B0=CD?=',), 'date': ('Sat, 20 Sep 2008 20:04:30 +0300 (\udcc3\udcb9\udcc3\udcb2\udcc3\udca5\udcc3\udcaf \udcc3\udcb7\udcc3\udca9\udcc3\udcb5 \udcc3\udca9\udcc3\udcb8\udcc3\udca5\udcc3\udcb9\udcc3\udcac\udcc3\udca9\udcc3\udcad)',)},
    attachments=[],
    from_values=EmailAddress(name='', email='mikel@me.com'),
    to_values=(EmailAddress(name='', email='bob@bob.com'),),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(),
)


import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='NOTE: 한국말로 하는 것',
    from_='jamis@37signals.com',
    to=('willard15georgina@jamis.backpackit.com',),
    cc=(),
    bcc=(),
    reply_to=(),
    date=datetime.datetime(2005, 5, 2, 16, 7, 5, tzinfo=datetime.timezone(datetime.timedelta(days=-1, seconds=64800))),
    date_str='Mon, 2 May 2005 16:07:05 -0600',
    text='대부분의 마찬가지로, 우리는 하나님을 믿습니다.\r\n\r\n제 이름은 Jamis입니다.',
    html='',
    headers={'mime-version': ('1.0 (Apple Message framework v622)',), 'content-transfer-encoding': ('base64',), 'message-id': ('<d3b8cf8e49f04480850c28713a1f473e@37signals.com>',), 'content-type': ('text/plain;\r\n  charset=EUC-KR;\r\n  format=flowed',), 'to': ('willard15georgina@jamis.backpackit.com',), 'from': ('Jamis Buck <jamis@37signals.com>',), 'subject': ('=?EUC-KR?Q?NOTE:_=C7=D1=B1=B9=B8=BB=B7=CE_=C7=CF=B4=C2_=B0=CD?=',), 'date': ('Mon, 2 May 2005 16:07:05 -0600',)},
    attachments=[],
    from_values=EmailAddress(name='Jamis Buck', email='jamis@37signals.com'),
    to_values=(EmailAddress(name='', email='willard15georgina@jamis.backpackit.com'),),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(),
)


import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='Meet the new MacBook Pro family. Now includes 13-inch.',
    from_='News@InsideApple.Apple.com',
    to=('karl.baum@gmail.com',),
    cc=(),
    bcc=(),
    reply_to=(),
    date=datetime.datetime(2009, 6, 11, 23, 25, 2, tzinfo=datetime.timezone(datetime.timedelta(days=-1, seconds=61200))),
    date_str='Thu, 11 Jun 2009 23:25:02 -0700',
    text="From one solid piece of aluminum comes a MacBook Pro that's thin and light, beautifully streamlined, and durable.\r\n",
    html='<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">\r\n<html>\r\n<head>\r\n<body>\r\nFrom one solid piece of aluminum comes a MacBook Pro that\'s thin and light, beautifully streamlined, and durable.<br><a href="http://insideapple.app</body>\r\n</html>\r\n',
    headers={'date': ('Thu, 11 Jun 2009 23:25:02 -0700',), 'from': ('Apple <News@InsideApple.Apple.com>',), 'to': ('karl.baum@gmail.com',), 'message-id': ('<7oh6b1$1clhrjk@badger-vip.apple.com>',), 'subject': ('Meet the new MacBook Pro family. Now includes 13-inch.',), 'mime-version': ('1.0',), 'content-type': ('multipart/alternative; boundary=mimepart_4b0c353551675_3d1c15b79ea5e70c1783',)},
    attachments=[],
    from_values=EmailAddress(name='Apple', email='News@InsideApple.Apple.com'),
    to_values=(EmailAddress(name='', email='karl.baum@gmail.com'),),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(),
)


import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='PGP Comments on RTO West Release of Dec. 14',
    from_='lpeters@PACIFIER.COM',
    to=('RRGA-L@LIST.RTOWEST.ORG',),
    cc=(),
    bcc=(),
    reply_to=(),
    date=datetime.datetime(2002, 1, 10, 13, 59, 53, tzinfo=datetime.timezone(datetime.timedelta(days=-1, seconds=57600))),
    date_str='Thu, 10 Jan 2002 13:59:53 -0800',
    text='Attached are the comments of the Public Generating Pool.\r\n--\r\n_________________________________\r\nLon L. Peters\r\nNorthwest Economic Research, Inc.\r\n6765 S.W. Preslynn Drive\r\nPortland, Oregon 97225-2668\r\n503-203-1539 (voice)\r\n503-203-1569 (fax)\r\n503-709-5942 (mobile)\r\nlpeters@pacifier.com\r\n\r\nNOTICE:  This communication and its attachments, if any, may contain\r\nsensitive, privileged, or other confidential information.  If you are\r\nnot the intended recipient or believe that you have received this\r\ncommunication in error, please notify the sender of this\r\ncommunication and delete the copy you received from all storage\r\ndevices.  In addition, please do not print, copy, retransmit,\r\nforward, disseminate, or otherwise use this communication or its\r\nattachments, if any.  Thank you.\r\n',
    html='',
    headers={'received': ('from nahou-mscnx06p.corp.enron.com ([192.168.110.237]) by napdx-msmbx01v.corp.enron.com with Microsoft SMTPSVC(5.0.2195.2966);\r\n\t Thu, 10 Jan 2002 14:12:53 -0800', 'from NAHOU-MSMSW06P.corp.enron.com ([192.168.110.228]) by nahou-mscnx06p.corp.enron.com with Microsoft SMTPSVC(5.0.2195.2966);\r\n\t Thu, 10 Jan 2002 16:12:52 -0600', 'from mailman.enron.com (unverified) by NAHOU-MSMSW06P.corp.enron.com\r\n (Content Technologies SMTPRS 4.2.5) with ESMTP id <T585d94ff81c0a86ee487c@NAHOU-MSMSW06P.corp.enron.com>;\r\n Thu, 10 Jan 2002 16:12:47 -0600', 'from tblexch01.transmission.bpa.gov (int.transmission.bpa.gov [206.137.58.133] (may be forged))\r\n\tby mailman.enron.com (8.11.4/8.11.4/corp-1.06) with ESMTP id g0AMCB920856;\r\n\tThu, 10 Jan 2002 16:12:11 -0600 (CST)', 'from TBLLIST1 ([206.137.58.134]) by tblexch01.transmission.bpa.gov with SMTP (Microsoft Exchange Internet Mail Service Version 5.5.2650.21)\r\n\tid CRWWC3K1; Thu, 10 Jan 2002 14:11:55 -0800', 'from LIST.TRANSMISSION.BPA.GOV by LIST.TRANSMISSION.BPA.GOV\r\n          (LISTSERV-TCP/IP release 1.8c) with spool id 2458 for\r\n          RRGA-L@LIST.TRANSMISSION.BPA.GOV; Thu, 10 Jan 2002 14:13:50 -0800', 'from [207.202.136.216] (ip136.r2.d.pdx.nwlink.com [207.202.136.136])\r\n          by comet.pacifier.com (8.11.2/8.11.1) with ESMTP id g0ALxLX06696;\r\n          Thu, 10 Jan 2002 13:59:21 -0800 (PST)'), 'mime-version': ('1.0',), 'x-sender': ('lpeters@mail.pacifier.com',), 'references': ('<1168BAF252B7D41194810001028D743108913C@SERVER>',), 'content-type': ('multipart/mixed;\r\n              boundary="============_-1201422494==_============"',), 'message-id': ('<p05100307b863befdfb67@[207.202.136.216]>',), 'date': ('Thu, 10 Jan 2002 13:59:53 -0800',), 'sender': ('RTO West Regional Representatives Group <RRGA-L@LIST.RTOWEST.ORG>',), 'from': ('"Lon L. Peters" <lpeters@PACIFIER.COM>',), 'subject': ('PGP Comments on RTO West Release of Dec. 14',), 'comments': ('cc: ltopaz@gcpud.org, gary.zarker@ci.seattle.wa.us,\r\n          dgodard@gcpud.org, wdobbins@dcpud.org, drobinson@cowlitzpud.org,\r\n          bgeddes@popud.com, "Culbertson, Tim" <tculber@gcpud.org>,\r\n          kknitte@gcpud.org, paula.green@ci.seattle.wa.us,\r\n          jim.harding@ci.seattle.wa.us, ghuhta@cowlitzpud.org,\r\n          dosborn@gcpud.org, jscheel@popud.com, jim.todd@ci.seattle.wa.us,\r\n          CWAGERS@dcpud.org, ali.rodol@ci.seattle.wa.us,\r\n          kevin.clark@ci.seattle.wa.us, bessex@cowlitzpud.org,\r\n          Cindy.Wright@ci.seattle.wa.us, "Juj, Hardev" \r\n          <Hardev.Juj@ci.seattle.wa.us>,\r\n          "Conger, Kurt" <kconger@energyexpertsvcs.com>,\r\n          "Kindley, Ray" <rkindley@schwabe.com>',), 'to': ('RRGA-L@LIST.RTOWEST.ORG',), 'in-reply-to': ('<1168BAF252B7D41194810001028D743108913C@SERVER>',), 'return-path': ('owner-rrga-l@list.rtowest.org',)},
    attachments=[
        dict(
            filename='PGP_Cmts_on_12-14-01_Pkg.doc',
            content_id='p05100307b863befdfb67@[207.202.136.216].0.0',
            content_disposition='attachment',
            content_type='application/msword',
            payload=b'',
        ),
        ],
    from_values=EmailAddress(name='Lon L. Peters', email='lpeters@PACIFIER.COM'),
    to_values=(EmailAddress(name='', email='RRGA-L@LIST.RTOWEST.ORG'),),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(),
)


import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='',
    from_='john.q.public@example.com',
    to=('mary@x.test', 'jdoe@example.org', 'one@y.test'),
    cc=('boss@nil.test', 'sysservices@example.net'),
    bcc=(),
    reply_to=(),
    date=datetime.datetime(2003, 7, 1, 10, 52, 37, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))),
    date_str='Tue, 1 Jul 2003 10:52:37 +0200',
    text='Hi everyone.\r\n',
    html='',
    headers={'from': ('"Joe Q. Public" <john.q.public@example.com>',), 'to': ('Mary Smith <mary@x.test>, jdoe@example.org, Who? <one@y.test>',), 'cc': ('<boss@nil.test>, "Giant; \\"Big\\" Box" <sysservices@example.net>',), 'date': ('Tue, 1 Jul 2003 10:52:37 +0200',), 'message-id': ('<5678.21-Nov-1997@example.com>',)},
    attachments=[],
    from_values=EmailAddress(name='Joe Q. Public', email='john.q.public@example.com'),
    to_values=(EmailAddress(name='Mary Smith', email='mary@x.test'), EmailAddress(name='', email='jdoe@example.org'), EmailAddress(name='Who?', email='one@y.test')),
    cc_values=(EmailAddress(name='', email='boss@nil.test'), EmailAddress(name='Giant; "Big" Box', email='sysservices@example.net')),
    bcc_values=(),
    reply_to_values=(),
)


# Lib author: Vladimir Kaukin <KaukinVK@ya.ru>
# Project home page: https://github.com/ikvk/imap_tools
# Mirror: https://gitflic.ru/project/ikvk/imap-tools
# License: Apache-2.0

from .query import AND, OR, NOT, Header, UidRange, A, O, N, H, U
from .mailbox import BaseMailBox, MailBox, MailBoxUnencrypted, MailBoxTls
from .message import MailMessage, MailAttachment
from .folder import MailBoxFolderManager, FolderInfo
from .consts import MailMessageFlags, MailBoxFolderStatusOptions, SortCriteria
from .utils import EmailAddress
from .errors import *

__version__ = '1.10.0'



import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='Re: Saying Hello',
    from_='mary@example.net',
    to=('jdoe@machine.example',),
    cc=(),
    bcc=(),
    reply_to=('smith@home.example',),
    date=datetime.datetime(1997, 11, 21, 10, 1, 10, tzinfo=datetime.timezone(datetime.timedelta(days=-1, seconds=64800))),
    date_str='Fri, 21 Nov 1997 10:01:10 -0600',
    text='This is a reply to your hello.\r\n',
    html='',
    headers={'from': ('Mary Smith <mary@example.net>',), 'to': ('John Doe <jdoe@machine.example>',), 'reply-to': ('"Mary Smith: Personal Account" <smith@home.example>',), 'subject': ('Re: Saying Hello',), 'date': ('Fri, 21 Nov 1997 10:01:10 -0600',), 'message-id': ('<3456@example.net>',), 'in-reply-to': ('<1234@local.machine.example>',), 'references': ('<1234@local.machine.example>',)},
    attachments=[],
    from_values=EmailAddress(name='Mary Smith', email='mary@example.net'),
    to_values=(EmailAddress(name='John Doe', email='jdoe@machine.example'),),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(EmailAddress(name='Mary Smith: Personal Account', email='smith@home.example'),),
)


import time
from imap_tools import MailBox, A

# 1
# SIMPLE
# waiting for msg in 60 sec, then print unseen if any update
with MailBox('imap.far.mars').login('acc', 'pwd') as mailbox:
    # in idle mode
    mailbox.idle.start()
    responses = mailbox.idle.poll(timeout=60)
    mailbox.idle.stop()
    # in not idle mode
    if responses:
        for msg in mailbox.fetch(A(seen=False)):
            print(msg.date, msg.subject)
    else:
        print('no any updates')

# 2
# reliable console notificator
# *some mail servers do not like multiple connections, you may close web mailbox interface for reduce connection errors
import time, socket, imaplib, traceback
from imap_tools import A, MailBox, MailboxLoginError, MailboxLogoutError

done = False
while not done:
    connection_start_time = time.monotonic()
    connection_live_time = 0.0
    try:
        with MailBox('imap.my.moon').login('acc', 'pwd', 'INBOX') as mailbox:
            print('@@ new connection', time.asctime())
            while connection_live_time < 29 * 60:
                try:
                    responses = mailbox.idle.wait(timeout=3 * 60)
                    print(time.asctime(), 'IDLE responses:', responses)
                    if responses:
                        for msg in mailbox.fetch(A(seen=False)):
                            print('->', msg.date, msg.subject)
                except KeyboardInterrupt:
                    print('~KeyboardInterrupt')
                    done = True
                    break
                connection_live_time = time.monotonic() - connection_start_time
    except (TimeoutError, ConnectionError,
            imaplib.IMAP4.abort, MailboxLoginError, MailboxLogoutError,
            socket.herror, socket.gaierror, socket.timeout) as e:
        print(f'## Error\n{e}\n{traceback.format_exc()}\nreconnect in a minute...')
        time.sleep(60)

# 3
# context manager
from imap_tools import MailBox, A

with MailBox('imap.sun.mw').login('acc', 'pwd', 'INBOX') as mailbox:
    with mailbox.idle as idle:
        responses = idle.poll(timeout=10)
    if responses:
        for msg in mailbox.fetch(A(seen=False)):
            print(msg.date, msg.subject)
    else:
        print('no any updates')



import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='Testing outlook',
    from_='email_test@me.nowhere',
    to=('mikel@me.nowhere',),
    cc=(),
    bcc=(),
    reply_to=('email_test@me.nowhere',),
    date=datetime.datetime(2007, 10, 21, 19, 38, 13, tzinfo=datetime.timezone(datetime.timedelta(seconds=36000))),
    date_str='Sun, 21 Oct 2007 19:38:13 +1000',
    text='Hello\r\nThis is an outlook test\r\n\r\nSo there.\r\n\r\nMe.\r\n',
    html='<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN">\r\n<HTML><HEAD>\r\n<META http-equiv=Content-Type content="text/html; charset=utf-8">\r\n<META content="MSHTML 6.00.6000.16525" name=GENERATOR>\r\n<STYLE></STYLE>\r\n</HEAD>\r\n<BODY bgColor=#ffffff>\r\n<DIV><FONT face=Arial size=2>Hello</FONT></DIV>\r\n<DIV><FONT face=Arial size=2><STRONG>This is an outlook \r\ntest</STRONG></FONT></DIV>\r\n<DIV><FONT face=Arial size=2><STRONG></STRONG></FONT>&nbsp;</DIV>\r\n<DIV><FONT face=Arial size=2><STRONG>So there.</STRONG></FONT></DIV>\r\n<DIV><FONT face=Arial size=2></FONT>&nbsp;</DIV>\r\n<DIV><FONT face=Arial size=2>Me.</FONT></DIV></BODY></HTML>\r\n\r\n',
    headers={'return-path': ('<email_test@me.nowhere>',), 'received': ('from omta05sl.mx.bigpond.com by me.nowhere.else with ESMTP id 632BD5758 for <mikel@me.nowhere.else>; Sun, 21 Oct 2007 19:38:21 +1000', 'from oaamta05sl.mx.bigpond.com by omta05sl.mx.bigpond.com with ESMTP id <20071021093820.HSPC16667.omta05sl.mx.bigpond.com@oaamta05sl.mx.bigpond.com> for <mikel@me.nowhere.else>; Sun, 21 Oct 2007 19:38:20 +1000', 'from mikel091a by oaamta05sl.mx.bigpond.com with SMTP id <20071021093820.JFMT24025.oaamta05sl.mx.bigpond.com@mikel091a> for <mikel@me.nowhere.else>; Sun, 21 Oct 2007 19:38:20 +1000'), 'date': ('Sun, 21 Oct 2007 19:38:13 +1000',), 'from': ('Mikel Lindsaar <email_test@me.nowhere>',), 'reply-to': ('Mikel Lindsaar <email_test@me.nowhere>',), 'to': ('mikel@me.nowhere',), 'message-id': ('<009601c813c6$19df3510$0437d30a@mikel091a>',), 'subject': ('Testing outlook',), 'mime-version': ('1.0',), 'content-type': ('multipart/alternative; boundary=----=_NextPart_000_0093_01C81419.EB75E850',), 'x-get_mail_default': ('mikel@me.nowhere.else',), 'x-priority': ('3',), 'x-original-to': ('mikel@me.nowhere',), 'x-mailer': ('Microsoft Outlook Express 6.00.2900.3138',), 'delivered-to': ('mikel@me.nowhere',), 'x-mimeole': ('Produced By Microsoft MimeOLE V6.00.2900.3138',), 'x-msmail-priority': ('Normal',)},
    attachments=[],
    from_values=EmailAddress(name='Mikel Lindsaar', email='email_test@me.nowhere'),
    to_values=(EmailAddress(name='', email='mikel@me.nowhere'),),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(EmailAddress(name='Mikel Lindsaar', email='email_test@me.nowhere'),),
)


import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='Testing outlook',
    from_='mikel@nowhere.com',
    to=('mikel@somewhere.com',),
    cc=(),
    bcc=(),
    reply_to=(),
    date=datetime.datetime(2007, 10, 21, 19, 38, 13, tzinfo=datetime.timezone(datetime.timedelta(seconds=36000))),
    date_str='Sun, 21 Oct 2007 19:38:13 +1000',
    text='Hello Mikel\r\n\r\n',
    html='',
    headers={'return-path': ('<mikel@nowhere.com>',), 'received': ('from mikel091a by oaamta05sl.mx.bigpond.com with SMTP id <20071021093820.JFMT24025.oaamta05sl.mx.bigpond.com@mikel091a> for <mikel@nowhere.com.else>; Sun, 21 Oct 2007 19:38:20 +1000',), 'date': ('Sun, 21 Oct 2007 19:38:13 +1000',), 'from': ('Mikel Lindsaar <mikel@nowhere.com>',), 'to': ('Mikel <mikel@somewhere.com>',), 'message-id': ('<009601c813c6$19df3510$0437d30a@mikel091a>',), 'subject': ('Testing outlook',)},
    attachments=[],
    from_values=EmailAddress(name='Mikel Lindsaar', email='mikel@nowhere.com'),
    to_values=(EmailAddress(name='Mikel', email='mikel@somewhere.com'),),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(),
)


import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='TTT: 1/20.1.UFA Depart',
    from_='bit@ufo.ru',
    to=('aaa@u6.ru', 'Serg@ufo.ru', 'isg@ggg.ru'),
    cc=(),
    bcc=(),
    reply_to=(),
    date=datetime.datetime(2021, 4, 20, 12, 31, 24, tzinfo=datetime.timezone(datetime.timedelta(seconds=18000))),
    date_str='Tue, 20 Apr 2021 12:31:24 +0500',
    text='MVT\r\nU68925/20.VQBCE.UFA\r\nAD0706/0729 EA1029 LBD\r\nDL/0126\r\n',
    html='',
    headers={'date': ('Tue, 20 Apr 2021 12:31:24 +0500',), 'from': ('<bit@ufo.ru>',), 'subject': ('TTT: 1/20.1.UFA Depart',), 'mime-version': ('1.0',), 'message-id': ('<2104200731.1@ufo.ru>',), 'content-transfer-encoding': ('Base64',), 'to': ('<aaa@u6.ru>, <Serg@ufo.ru>, <isg@ggg.ru>',), 'content-type': ('multipart/mixed; charset="utf-8";\r\n\tboundary="96b0f72f_a01e_44f0_8595_352ca3ba6fd4"',)},
    attachments=[],
    from_values=EmailAddress(name='', email='bit@ufo.ru'),
    to_values=(EmailAddress(name='', email='aaa@u6.ru'), EmailAddress(name='', email='Serg@ufo.ru'), EmailAddress(name='', email='isg@ggg.ru')),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(),
)


"""
Query builder examples.

NOTES:

# Infix notation (natural to humans)
    NOT ((FROM='11' OR TO="22" OR TEXT="33") AND CC="44" AND BCC="55")
# Prefix notation (Polish notation, IMAP version)
    NOT (((OR OR FROM "11" TO "22" TEXT "33") CC "44" BCC "55"))
# Python query builder
    NOT(AND(OR(from_='11', to='22', text='33'), cc='44', bcc='55'))

# python to prefix notation steps:
1. OR(1=11, 2=22, 3=33) ->
    "(OR OR FROM "11" TO "22" TEXT "33")"
2. AND("(OR OR FROM "11" TO "22" TEXT "33")", cc='44', bcc='55') ->
    "AND(OR(from_='11', to='22', text='33'), cc='44', bcc='55')"
3. NOT("AND(OR(from_='11', to='22', text='33'), cc='44', bcc='55')") ->
    "NOT (((OR OR FROM "1" TO "22" TEXT "33") CC "44" BCC "55"))"
"""

import datetime as dt
from imap_tools import AND, OR, NOT, A, H, U

# date in the date list (date=date1 OR date=date3 OR date=date2)
q1 = OR(date=[dt.date(2019, 10, 1), dt.date(2019, 10, 10), dt.date(2019, 10, 15)])
# '(OR OR ON 1-Oct-2019 ON 10-Oct-2019 ON 15-Oct-2019)'

# date not in the date list (NOT(date=date1 OR date=date3 OR date=date2))
q2 = NOT(OR(date=[dt.date(2019, 10, 1), dt.date(2019, 10, 10), dt.date(2019, 10, 15)]))
# 'NOT ((OR OR ON 1-Oct-2019 ON 10-Oct-2019 ON 15-Oct-2019))'

# subject contains "hello" AND date greater than or equal dt.date(2019, 10, 10)
q3 = A(subject='hello', date_gte=dt.date(2019, 10, 10))
# '(SUBJECT "hello" SINCE 10-Oct-2019)'

# from contains one of the address parts
q4 = OR(from_=["@spam.ru", "@tricky-spam.ru"])
# '(OR FROM "@spam.ru" FROM "@tricky-spam.ru")'

# marked as seen and not flagged
q5 = AND(seen=True, flagged=False)
# '(SEEN UNFLAGGED)'

# (text contains tag15 AND subject contains tag15) OR (text contains tag10 AND subject contains tag10)
q6 = OR(AND(text='tag15', subject='tag15'), AND(text='tag10', subject='tag10'))
# '(OR (TEXT "tag15" SUBJECT "tag15") (TEXT "tag10" SUBJECT "tag10"))'

# (text contains tag15 OR subject contains tag15) OR (text contains tag10 OR subject contains tag10)
q7 = OR(OR(text='tag15', subject='tag15'), OR(text='tag10', subject='tag10'))
# '(OR (OR TEXT "tag15" SUBJECT "tag15") (OR TEXT "tag10" SUBJECT "tag10"))'

# header IsSpam contains '++' AND header CheckAntivirus contains '-'
q8 = A(header=[H('IsSpam', '++'), H('CheckAntivirus', '-')])
# '(HEADER "IsSpam" "++" HEADER "CheckAntivirus" "-")'

# UID range
q9 = A(uid=U('1034', '*'))
# '(UID 1034:*)'

# complex from README
q10 = A(OR(from_='from@ya.ru', text='"the text"'), NOT(OR(A(answered=False), A(new=True))), to='to@ya.ru')
# '((OR FROM "from@ya.ru" TEXT "\\"the text\\"") NOT ((OR (UNANSWERED) (NEW))) TO "to@ya.ru")'



import re
import sys

SHORT_MONTH_NAMES = ('Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec')

UID_PATTERN = re.compile(r'(^|\s+|\W)UID\s+(?P<uid>\d+)')

CODECS_OFFICIAL_REPLACEMENT_CHAR = '�'

PYTHON_VERSION_MINOR = int(sys.version_info.minor)

MOVE_RESULT_TAG = ('_MOVE',)  # const delete_result part for mailbox.move result, when server have MOVE in capabilities

class MailMessageFlags:
    """
    System email message flags
    All system flags begin with "\"
    """
    SEEN = '\\Seen'
    ANSWERED = '\\Answered'
    FLAGGED = '\\Flagged'
    DELETED = '\\Deleted'
    DRAFT = '\\Draft'
    RECENT = '\\Recent'
    all = (
        SEEN, ANSWERED, FLAGGED, DELETED, DRAFT, RECENT
    )


class MailBoxFolderStatusOptions:
    """Valid mailbox folder status options"""
    MESSAGES = 'MESSAGES'
    RECENT = 'RECENT'
    UIDNEXT = 'UIDNEXT'
    UIDVALIDITY = 'UIDVALIDITY'
    UNSEEN = 'UNSEEN'
    all = (
        MESSAGES, RECENT, UIDNEXT, UIDVALIDITY, UNSEEN
    )
    description = (
        (MESSAGES, "The number of messages in the mailbox"),
        (RECENT, "The number of messages with the Recent flag set"),
        (UIDNEXT, "The next unique identifier value of the mailbox"),
        (UIDVALIDITY, "The unique identifier validity value of the mailbox"),
        (UNSEEN, "The number of messages which do not have the Seen flag set"),
    )


class SortCriteria:
    """
    Sort criteria
    https://datatracker.ietf.org/doc/html/rfc5256
    ARRIVAL - Internal date and time of the message.
        This differs from the ON criteria in SEARCH, which uses just the internal date.
    CC - [IMAP] addr-mailbox of the first "cc" address.
    DATE - Sent date and time, as described in section 2.2.
    FROM - [IMAP] addr-mailbox of the first "From" address.
    SIZE - Size of the message in octets.
    SUBJECT - Base subject text.
    TO - [IMAP] addr-mailbox of the first "To" address.
    """
    ARRIVAL_DT_ASC = 'ARRIVAL'
    CC_ASC = 'CC'
    DATE_ASC = 'DATE'
    FROM_ASC = 'FROM'
    SIZE_ASC = 'SIZE'
    SUBJECT_ASC = 'SUBJECT'
    TO_ASC = 'TO'

    ARRIVAL_DT_DESC = 'REVERSE ARRIVAL'
    CC_DESC = 'REVERSE CC'
    DATE_DESC = 'REVERSE DATE'
    FROM_DESC = 'REVERSE FROM'
    SIZE_DESC = 'REVERSE SIZE'
    SUBJECT_DESC = 'REVERSE SUBJECT'
    TO_DESC = 'REVERSE TO'

    all = (
        ARRIVAL_DT_ASC, CC_ASC, DATE_ASC, FROM_ASC, SIZE_ASC, SUBJECT_ASC, TO_ASC,
        ARRIVAL_DT_DESC, CC_DESC, DATE_DESC, FROM_DESC, SIZE_DESC, SUBJECT_DESC, TO_DESC,
    )



import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='Redacted',
    from_='redacted@example.com',
    to=('redacted@example.com',),
    cc=(),
    bcc=(),
    reply_to=(),
    date=datetime.datetime(2002, 1, 22, 14, 35, 28, tzinfo=datetime.timezone.utc),
    date_str='Tue, 22 Jan 2002 14:35:28 UT',
    text='',
    html='',
    headers={'message-id': ('<200201221435.g0MEZP927213@mailman.enron.com>',), 'content-type': ('multipart/related; boundary="_----------=_10117101281980"',), 'mime-version': ('1.0',), 'date': ('Tue, 22 Jan 2002 14:35:28 UT',), 'subject': ('Redacted',), 'to': ('redacted@example.com',), 'from': ('redacted@example.com',), 'return-path': ('redacted@example.com',)},
    attachments=[],
    from_values=EmailAddress(name='', email='redacted@example.com'),
    to_values=(EmailAddress(name='', email='redacted@example.com'),),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(),
)


from imap_tools import MailBox, MailMessage

# parse .eml files attached to email messages
with MailBox('imap.mail.com').login('test@mail.com', 'password') as mailbox:
    for message in mailbox.fetch():
        for att in message.attachments:
            if '.eml' in att.filename:
                print(att.filename)
                eml_email = MailMessage.from_bytes(att.payload)
                print('- subject: ', eml_email.subject)
                print('- date_str: ', eml_email.date_str)

# parse .eml files from attached .eml files (2 levels of nesting)
with open('/tmp/1.eml', 'rb') as f:
    bytes_data = f.read()
msg = MailMessage.from_bytes(bytes_data)
for att in msg.attachments:
    if '.eml' in att.filename:
        # 1 level .eml
        msg_level_1 = MailMessage.from_bytes(att.payload)
        print(att.filename)
        print('* subject: ', msg_level_1.subject)
        print('* date: ', msg_level_1.date)
        for att_sub in msg_level_1.attachments:
            if '.eml' in att_sub.filename:
                # 2 level .eml
                msg_level_2 = MailMessage.from_bytes(att_sub.payload)
                print('|--', att_sub.filename)
                print('|   * subject: ', msg_level_2.subject)
                print('|   * date: ', msg_level_2.date)



import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='eBay Bid - 2008 Ford Super Duty F-350 DRW King Ranch Crew Cab 4x4 (156) - Stock# XXXX',
    from_='no-reply@crm.el-example.org',
    to=('e-f5f4@app.ar-example.com',),
    cc=(),
    bcc=(),
    reply_to=('',),
    date=datetime.datetime(2010, 9, 22, 2, 30, 53, tzinfo=datetime.timezone(datetime.timedelta(days=-1, seconds=68400))),
    date_str='Wed, 22 Sep 2010 02:30:53 -0500',
    text='Body Text',
    html='',
    headers={'delivered-to': ('e-f5f4@app.ar-example.com',), 'received': ('by 10.2.1.1 with SMTP id p21cs62610wem;\r\n        Wed, 22 Sep 2010 00:30:55 -0700 (PDT)', 'by 10.15.2.1 with SMTP id p24mr499372ybh.380.1285140655013;\r\n        Wed, 22 Sep 2010 00:30:55 -0700 (PDT)', 'from aquila.el-example.org ([174.1.8.2])\r\n        by mx.google.com with ESMTP id a6si11272839ybo.18.2010.09.22.00.30.54;\r\n        Wed, 22 Sep 2010 00:30:54 -0700 (PDT)', 'from aquila.el-example.org (localhost [127.0.0.1])\r\n\tby aquila.el-example.org (8.14.2/8.14.2) with ESMTP id o8M7UrfD018673\r\n\tfor <e-f5f4@app.ar-example.com>; Wed, 22 Sep 2010 02:30:54 -0500 (CDT)\r\n\t(envelope-from production@aquila.el-example.org)', '(from production@localhost)\r\n\tby aquila.el-example.org (8.14.2/8.14.2/Submit) id o8M7Urh3018672;\r\n\tWed, 22 Sep 2010 02:30:53 -0500 (CDT)\r\n\t(envelope-from production)'), 'return-path': ('<production@aquila.el-example.org>',), 'received-spf': ('neutral (google.com: 174.1.8.2 is neither permitted nor denied by best guess record for domain of production@aquila.el-example.org) client-ip=174.1.8.2;',), 'authentication-results': ('mx.google.com; spf=neutral (google.com: 174.1.8.2 is neither permitted nor denied by best guess record for domain of production@aquila.el-example.org) smtp.mail=production@aquila.el-example.org',), 'message-id': ('<201009220730.o8M7Urh3018672@aquila.el-example.org>',), 'mime-version': ('1.0',), 'content-disposition': ('inline',), 'content-transfer-encoding': ('quoted-printable',), 'content-type': ('text/plain',), 'x-mailer': ('MIME::Lite 3.027 (F2.74; T1.28; A2.04; B3.07; Q3.07)',), 'date': ('Wed, 22 Sep 2010 02:30:53 -0500',), 'from': ('no-reply@crm.el-example.org',), 'to': ('e-f5f4@app.ar-example.com',), 'reply-to': ('"KLAUS- H\udcc3\udc84NSCHEL" <>',), 'subject': ('eBay Bid - 2008 Ford Super Duty F-350 DRW King Ranch Crew Cab 4x4 (156) - Stock# XXXX',)},
    attachments=[],
    from_values=EmailAddress(name='', email='no-reply@crm.el-example.org'),
    to_values=(EmailAddress(name='', email='e-f5f4@app.ar-example.com'),),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(EmailAddress(name='KLAUS- HÄNSCHEL', email=''),),
)


import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='testing',
    from_='foo@example.com',
    to=('blah@example.com',),
    cc=(),
    bcc=(),
    reply_to=(),
    date=datetime.datetime(2005, 6, 6, 22, 21, 22, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))),
    date_str='Mon, 6 Jun 2005 22:21:22 +0200',
    text='This is the first part.\r\n\r\n',
    html='',
    headers={'mime-version': ('1.0 (Apple Message framework v730)',), 'content-type': ('multipart/mixed; boundary=Apple-Mail-13-196941151',), 'message-id': ('<9169D984-4E0B-45EF-82D4-8F5E53AD7012@example.com>',), 'from': ('foo@example.com',), 'subject': ('testing',), 'date': ('Mon, 6 Jun 2005 22:21:22 +0200',), 'to': ('blah@example.com',)},
    attachments=[
        dict(
            filename='test.rb',
            content_id='',
            content_disposition='attachment',
            content_type='text/x-ruby-script',
            payload=b'puts "testing, testing"\r\n',
        ),
        
        dict(
            filename='test.pdf',
            content_id='',
            content_disposition='inline',
            content_type='application/pdf',
            payload=b'blah blah blah',
        ),
        
        dict(
            filename='smime.p7s',
            content_id='',
            content_disposition='attachment',
            content_type='application/pkcs7-signature',
            payload=b"\x8d\xa9\xa2\xb1*\x86H\x86\xf7\r\x01\x07\x02\xa0\x800\x88\xda\x9a+1\x0b0\t\x06\x05+\x0e\x03\x02\x1a\x05\x000\x80\x06\t*\x86J6\xa6\x8a\xc1\x07\x01\x00\x00\xa0\x82\x05J0\x82\x05F0\x82\x04.\x8d\xa9\xa2\xb1\x02\x02\x04?\xbe\xbaD0\r\x06\t*\x88\xda\x9a+\r\x01\x01\x05\x05\x00011\x0b0\t\x06\x03U\x04\x06\x13\x02F6\xa6\x8a\xc0\n\x06\x03U\x04\n\x13\x03TDC1\x140\x12\x06\x8d\xa9\xa2\xb3\x13\x0bTDC OCES CH\xda\x9a+\r040229115901Z\x17\r06026\xa6\x8a\xc22901Z0\x81\x801\x0b0\t\x06\x03U\x04\x8d\xa9\xa2\xb0K1)0'\x06\x03U\x04\n\x13 H\xda\x9a+. organisatorisk tin6\xa6\x8a\xc4nin",
        ),
        ],
    from_values=EmailAddress(name='', email='foo@example.com'),
    to_values=(EmailAddress(name='', email='blah@example.com'),),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(),
)


import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='Re: Test reply email',
    from_='xxxxxxxx@xxx.org',
    to=('mikel@xxxx.net',),
    cc=(),
    bcc=(),
    reply_to=(),
    date=datetime.datetime(2007, 11, 18, 19, 56, 7, tzinfo=datetime.timezone(datetime.timedelta(seconds=39600))),
    date_str='Sun, 18 Nov 2007 19:56:07 +1100',
    text='Message body\r\n',
    html='',
    headers={'return-path': ('<xxxxxxxx@xxx.org>',), 'received': ('from me ([unix socket])\r\n\t by xxxxx1.xxxx.net (Cyrus v2.2.12) with LMTPA;\r\n\t Sun, 18 Nov 2007 00:56:33 -0800', 'from smtp.xxxx.org (unknown [127.0.0.1])\r\n\tby xxxxx1.xxxx.net (Postfix) with ESMTP id F128477EB5;\r\n\tSun, 18 Nov 2007 00:56:32 -0800 (PST)', 'from omta02sl.mx.bigpond.com (omta02sl.mx.bigpond.com [144.140.93.154])\r\n\tby smtp.xxxx.org (Postfix) with ESMTP id 2D567ACC08;\r\n\tSun, 18 Nov 2007 00:56:28 -0800 (PST)', 'from oaamta05sl.mx.bigpond.com ([124.183.219.10])\r\n          by omta02sl.mx.bigpond.com with ESMTP\r\n          id <20071118085627.YVPI22254.omta02sl.mx.bigpond.com@oaamta05sl.mx.bigpond.com>;\r\n          Sun, 18 Nov 2007 08:56:27 +0000', 'from [10.0.0.1] (really [124.183.219.10])\r\n          by oaamta05sl.mx.bigpond.com with ESMTP\r\n          id <20071118085627.TQWF6995.oaamta05sl.mx.bigpond.com@[10.0.0.1]>;\r\n          Sun, 18 Nov 2007 08:56:27 +0000'), 'message-id': ('<473FFE27.20003@xxx.org>',), 'date': ('Sun, 18 Nov 2007 19:56:07 +1100',), 'from': ('Testing <xxxxxxxx@xxx.org>',), 'user-agent': ('Mozilla Thunderbird 1.0.6 (Windows/20050716)',), 'x-accept-language': ('en-us, en',), 'mime-version': ('1.0',), 'to': ('Mikel Lindsaar <mikel@xxxx.net>',), 'subject': ('Re: Test reply email',), 'references': ('<473FF3B8.9020707@xxx.org> <348F04F142D69C21-291E56D292BC@xxxx.net>',), 'in-reply-to': ('<348F04F142D69C21-291E56D292BC@xxxx.net>',), 'content-type': ('text/plain; charset=US-ASCII; format=flowed',), 'content-transfer-encoding': ('7bit',)},
    attachments=[],
    from_values=EmailAddress(name='Testing', email='xxxxxxxx@xxx.org'),
    to_values=(EmailAddress(name='Mikel Lindsaar', email='mikel@xxxx.net'),),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(),
)


import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='Re: TESTテストテスト',
    from_='atsushi@example.com',
    to=('rudeboyjet@gmail.com',),
    cc=(),
    bcc=(),
    reply_to=('rudeboyjet@gmail.com',),
    date=datetime.datetime(2011, 8, 19, 10, 47, 17, tzinfo=datetime.timezone(datetime.timedelta(seconds=32400))),
    date_str='Fri, 19 Aug 2011 10:47:17 +0900',
    text='Hello\r\n',
    html='',
    headers={'date': ('Fri, 19 Aug 2011 10:47:17 +0900',), 'from': ('Atsushi Yoshida <atsushi@example.com>',), 'reply-to': ('rudeboyjet@gmail.com',), 'subject': ('Re: TEST\r\n \r\n\t=?ISO-2022-JP?B?GyRCJUYlOSVIGyhC?=\r\n  =?ISO-2022-JP?B?GyRCJUYlOSVIGyhC?=',), 'to': ('rudeboyjet@gmail.com',), 'message-id': ('<0CC5E11ED2C1D@example.com>',), 'in-reply-to': ('<rid_5582199198@msgid.example.com>',), 'mime-version': ('1.0',), 'content-type': ('text/plain; charset=iso-2022-jp',), 'content-transfer-encoding': ('7bit',)},
    attachments=[],
    from_values=EmailAddress(name='Atsushi Yoshida', email='atsushi@example.com'),
    to_values=(EmailAddress(name='', email='rudeboyjet@gmail.com'),),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(EmailAddress(name='', email='rudeboyjet@gmail.com'),),
)


import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='Mail.Ru – больше, чем почта. Познакомьтесь с проектами Mail.Ru Group',
    from_='welcome@corp.mail.ru',
    to=('imap.tools@mail.ru',),
    cc=(),
    bcc=(),
    reply_to=(),
    date=datetime.datetime(1900, 1, 1, 0, 0),
    date_str='',
    text='Здравствуйте\r\n',
    html='<html>\r\n  <head>\r\n    <meta http-equiv="content-type" content="text/html; charset=utf-8">\r\n  </head>\r\n  <body bgcolor="#FFFFFF" text="#000000">\r\n    <meta http-equiv="Content-Type" content="text/html; charset=utf-8">\r\n        \r\n    <div style="line-height:0;font-size:0;"><img\r\n                src="cid:part1.02000503.06000807@mail.ru" alt=""\r\n                style="width:28px;height:1px;" height="1" width="28"></div>\r\n\r\n    <div style="line-height:0;height:25px;" class="mmwe3-mso_lh5"><img\r\n                src="cid:part51.09060808.01070700@mail.ru"\r\n                style="width:125px;height:25px;" alt="@Mail.Ru"\r\n                height="25" width="125"></div>\r\n                \r\n    <div style="line-height:0;height:60px;" class="mmwe3-mso_lh4"><a\r\n                        href="https://www.odnoklassniki.ru/pochtamail"\r\n                        title="Одноклассники"><img\r\n                          src="cid:part41.01020108.09060604@mail.ru"\r\n                          style="width:60px;height:60px;"\r\n                          alt="Одноклассники" height="60" width="60"></a></div>\r\n\r\n  </body>\r\n</html>\r\n',
    headers={'to': ('imap.tools@mail.ru',), 'mime-version': ('1.0',), 'from': ('=?UTF-8?B?0JrQvtC80LDQvdC00LAg0J/QvtGH0YLRiyBNYWlsLlJ1?=\r\n <welcome@corp.mail.ru>',), 'subject': ('=?UTF-8?B?TWFpbC5SdSDigJMg0LHQvtC70YzRiNC1LCDRh9C10Lwg0L/QvtGH0YLQsC4g?=\r\n =?UTF-8?B?0J/QvtC30L3QsNC60L7QvNGM0YLQtdGB0Ywg0YEg0L/RgNC+0LXQutGC0LA=?=\r\n =?UTF-8?B?0LzQuCBNYWlsLlJ1IEdyb3Vw?=',), 'content-type': ('multipart/alternative;\r\n boundary="------------080400030209020702000207"',)},
    attachments=[
        dict(
            filename='blank.png',
            content_id='part1.02000503.06000807@mail.ru',
            content_disposition='inline',
            content_type='image/png',
            payload=b'\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00\x08\x00\x00\x00\x08\x01\x00\x00\x00\x00\xect\x83&\x00\x00\x00\x02tRNS\x00\x01\x01\x94\xfd\xae\x00\x00\x00\x0bIDATx\x01c\xfc\x8f\n\x01@\x18\x08\x01\xdez\xda\x08\x00\x00\x00\x00IEND\xaeB`\x82',
        ),
        
        dict(
            filename='sm-ok.png',
            content_id='part41.01020108.09060604@mail.ru',
            content_disposition='inline',
            content_type='image/png',
            payload=b'\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00<\x00\x00\x00<\x08\x03\x00\x00\x00\r")@\x00\x00\x00\x19tEXtSoftware\x00Adobe ImageReadyq\xc9e<\x00\x00\x01\xd1PLTE\xffs\x01\xfft\x03\xffu\x06\xff\xf7\xf0\xff\x84!\xff\x93<\xff\xf3\xe9\xff\xfc\xf9\xff\x94>\xff\xde\xc3\xff\xdd\xc1\xff\xf5\xed\xff\xda\xbc\xff\xc8\x9b\xffs\x02\xff\xf8\xf3\xff\xael\xff\x906\xff\xfc\xfa\xff\xef\xe2\xff\xf0\xe3\xff\x8b-\xff\xf4\xec\xff\xa2W\xff\x94=\xff\x81\x1c\xff\xc8\x9c\xff\xeb\xda\xff\xd7\xb7\xfft\x04\xff\xba\x83\xff\xcc\xa2\xff\x9fQ\xff\xd1\xac\xff\xfd\xfb\xff\xfb\xf8\xff\xf6\xef\xff\xd0\xaa\xff\xa3Y\xff\x92:\xffy\x0c\xff{\x10\xff\xbd\x87\xff\xee\xe1\xff\xf3\xea\xffv\x08\xff\xeb\xdb\xff\x9bK\xff\xa2V\xff\xc9\x9d\xff\xc1\x8f\xff\x9cL\xff\xf7\xf1\xff\xfa\xf6\xff\x8f5\xff\xf1\xe6\xff\xee\xe0\xff\xd7\xb6\xff\xc9\x9e\xff\xa3X\xff\xc7\x99\xff\xadj\xffu\x05\xff\x8e3\xff\xadk\xffz\x0f\xff\xa0S\xff{\x11\xff\xdd\xc2\xff\xf8\xf2\xff\x929\xff\xdc\xbf\xff\x93;\xff\x8e2\xff\xdc\xc0\xff\x97C\xff\xcf\xa9\xff\xba\x82\xff\xc7\x9a\xffy\r\xff\x83\x1e\xff\xa4[\xff\xa8b\xff\x8d0\xff|\x12\xff\x88\'\xff\xb7}\xff\xc3\x93\xff\x9fR\xff\xed\xdf\xff\x86%\xff\x96B\xff\x8c/\xff\xd3\xb0\xff\xca\xa0\xff\xa9c\xff\xaci\xff\xe8\xd5\xff\xbf\x8c\xff\xed\xde\xff\xaem\xff\x918\xffv\x07\xff\xbf\x8b\xff\xdb\xbe\xffx\n\xff\xafo\xff\xc3\x92\xff\x95?\xff\xca\x9f\xff\xbd\x88\xff\xc6\x98\xff\x81\x1b\xff\xd6\xb4\xff\x9aH\xff\xec\xdd\xff\xd3\xaf\xff\xd9\xbb\xff\xf9\xf5\xff\x8d1\xff\xf9\xf4\xff\xc5\x96\xff\x907\xff\xf0\xe4\xff\xa9d\xff\xaae\xff\x8a,\xff\xd2\xae\xff\x95@\xff\xfd\xfc\xff\x83\x1f\xff\xd6\xb5\xff\xd8\xb8\xff\xa0T\xff\xc5\x97\xff\xbe\x89\xff\xbe\x8a\xff}\x14\xff\x96A\xff\xa4Z\xffz\x0e\xff\xf6\xee\xff\x84 \xff\xce\xa7\xff\xd4\xb2\xff\x9aI\xff\xcd\xa5\xff}\x13\xff\xf2\xe7\xff\xa1U\xff\x8a+\xff\xbb\x84\xff\xd4\xb1\xffr\x00\xff\xff\xff\xe4\xd8\x99$\x00\x00\x03\xcbIDATx\xda\xac\x97g_\x13A\x10\x877\xb9\x90J\x08\x81P\x12i!\x14)J/J\x17\xa5*\xa0" \x08*J\xb5\xf7\xde{\xefe\xf6\xd3:sg\x80d\xe7\x92X\xe6\r\xfbc\xe6\xc9\xee\xed\xee\xfcgVH\xd6\xb4\x82\xa2\x8e\xdb\xc1\xed\x00\xdb\x83\xfe\x8e\xa2\x02\x8d\x8f\x12\xcc\xff\x02\x8b\x95\x16\x881K\xe5bMJ\xb0\xbb)\x8e\xfc\xcd7\x8d$\x85\x07\xf3\xf5\xd0\x8c\xe1\xcc\x90\xab\xdd\'\xa5\xaf\xdd\x15\xca\x1c\xce\xd0\xff\x99?\x98\x10n;HA\xc2\xd9Y\x1b\x1bU\xdb\xe9\x14\xe49\xd8f\x0e\x97\xf7a\x80\xf5\xf86ns\xb6\x1d\xb7\xa2\xb3\xaf\xdc\x04vd\xd2\x8f\xe7\xd5H\x13\xcb\xee \x7ff\x84\x83\xf7\x8d\xa2\xcb\x96.\x13X\xba\rC\x9c\x1e\x15\xae\xedAGI\x99Lhe3\x18t\xac6\x1e\xd6\x86q\xa3\xce\xcb\xa4v\x1e7\xee\x86\x16\x0b;\xe6\xf0$\x8be\nV\x8c\xb7`\xce\x11\x03w\xe1\xbc\x9d2%\xeb\xc4\xb9\xbb\xb6\xc2\xe5\xf8)\xd52E\xab\xc6\xe0\xfd\x9b\xf0\x9d*\x80\x19\x99\xb2\xed\x01\xa8\nl\xc0%xFZ\xea\xb0\x86\'V\x12\x85Wq\x1d\xe9\xf2\x0f,\xfd7@p\x1a@\x05\x1btr~\xfe$\xeb\xa8\x00H3`7\xc0\x8el5\xc0q\xe6=]G\xdb\x99,\xe6\xa6\xee\x00p\xebp\x13@\xaf\xea\xffZ\x19\xcd\xe4!\xbb\xea\xed\x05h"\xd8^\x08B\xcd\xa3\x96c\x9b:p\xb8Eq\xef\x14PhG\xb8\x11\xb3\\\xfd\xe9)C\x13\x0c\x15\x98R\xfdN\x80F\x84q\xbb\xc6\x14_V?2\xe3\xc5\x91H\xf18\x0e&\xd5\xcf\xf6\xd2\x96\t\xcd\x02B\xfd(7M\xf8\x81F\x9fi\xb4W\t\xb0\x0b\xb0hb\x17@\x8e\xba\xaa\xddH,\xeb\xf3e\x91\x80,\xa8\x119\x00\xbb\xc4\x0f\x80\xd3\xaak\x8c\xf4H\x87\x1d\x04{\xd5\x08\x94\x9d\x03"\x0f $\xf9e\x9f\x8b\xae\x81Y\xb6l@\xc5\x12~\x00\x97\xea*\xfd\x82\xc8\xda\xf3\xe6\xe6[k8x\xf5H\x8dp\x01\xf8E\x10\xe0(s\x01\x0fl\x15\xfc"&\xe0\x14\xc0E\x81\xf5h\x1f\xe3s\xa4m\xb2\xf9\x0e& \x17\x8f\x92\xb4\x9c\xbd\xfb\x9e\xd1(;\xea\xe1\xfc\x0e\xaa\x0ef\xb0,\r}\'\xf4~C\xa94\x81\xad\xb4l\x1f3\xef\xb902\xd9{]\x98n\xa5\xe1\x05\x0f\xbb\xecI~\xc3|\x13\x00\xb3\r:\xe2\t\xcd\x02L\xf8\xb8\r{\xcc\x1f\xd5\x13#-\x8e\x9c={\xc4H\x8d\x02\xfe\xa8\xf0\x92\xecV\xabZ\\\x89.\xdc\xc9]\xe0<Q\xc4^\xcfpp+\x1b\x0c\x9b\\O>1d\xcbtO\x14\xed\x99n\x91&\x89\xc1\xa7$\xd9\x05\x83\x1dc\x9d\x94\x92e\xba\x18x9\xf7G\xab\x01/_\xe2\xbc\x86\x18\x90\x0c9\x19\xefUkt\xd9\xd6\x9f\xd2T\x86H\x00\xd5\xcd\x0c\xeb\xac\xbfU\xa7\x1f\x98\n\xa0\xac\xc3^!\xdeYPHT\xa5\xa6\r\xd0\xdf\xee\xa7\xcc^\xd7m\x88~ \xcey\x99\x984L\xb7\xe6!\x1a}2\x15}\xae\xdc\xd4S\xf3\xa1\xdfI\x1f5f\xf5\xa6\xe5f\xa3nm\xb1\xf6%k\xc7ucx3\xcf\xba\x94\xad\x16\xbak\xff\xa3\xc4\xca\x00\x16\xf7=\x7f[\xdc\xe5\xfe\x7fh+\xa4<\x81\x9a\xe2M\x8d\xf5\xa2\xf8\x9c\x88m\xa5\x1e\xfeI+\xe5\x8cm\xa5d.f\x89x\x91\x9c}\x89\xf3\xe6\xe4\xc6\xb7\x8f\xf6\xc3\xf8)\xcf\x92\xb5\x8f\xdf0(\xc7\xae6\xae\xcd\xaf\xa9\x89p\'b\xdd\xd4\xb8\xe6{\xb8\x969\xd2KR|(`\x86\x06\x0e\x91NwE\x126\xebw\xf9f\xfd\x1e\xe5\xd9\xdb\xf2\x04\xcf\x847\xc63\xc1\x9b\x1b\'\xd2^\xe3\x99P\xd1\x96\xf8\x81b\xd4(K\xeb\xba\xfe@\xc9zw\xc5\xd5\xb0\xee7\xc4th5\xe9\xd3h\xa4.\x83{\x1ae\xd4%\x7f\x1a\x91\xd5\xac\x0c\xc4?\xca\x06VR{\x94E\x9f\x83\xf5\xad\xb6\xfe\xee\xee~[k\xbd\xe9s\xf0\x97\x00\x03\x003\xa2}\xdf\xee~*\xa5\x00\x00\x00\x00IEND\xaeB`\x82',
        ),
        
        dict(
            filename='logo.png',
            content_id='part51.09060808.01070700@mail.ru',
            content_disposition='inline',
            content_type='image/png',
            payload=b'\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00}\x00\x00\x00\x19\x08\x03\x00\x00\x00+;\xd1[\x00\x00\x00\x19tEXtSoftware\x00Adobe ImageReadyq\xc9e<\x00\x00\x00fPLTE\xfd\xfd\xfd\xfe\xfe\xfe\xf3\xf3\xf3\xec\xec\xec\xe8\xe8\xe8\xdf\xdf\xdf\xf1\xf1\xf1\xfc\xfc\xfc\xee\xee\xee\xe2\xe2\xe2\xfb\xfb\xfb\xf7\xf7\xf7\xe0\xe0\xe0\xf5\xf5\xf5\xed\xed\xed\xf2\xf2\xf2\xf9\xf9\xf9\xeb\xeb\xeb\xe6\xe6\xe6\xf4\xf4\xf4\xef\xef\xef\xf6\xf6\xf6\xf8\xf8\xf8\xe7\xe7\xe7\xe3\xe3\xe3\xe5\xe5\xe5\xea\xea\xea\xfa\xfa\xfa\xe4\xe4\xe4\xe1\xe1\xe1\xf0\xf0\xf0\xe9\xe9\xe9\xde\xde\xde\xff\xff\xff\xe4X/e\x00\x00\x03-IDATx\xda\xc4\x96\xd9\x96\x83 \x0c\x86\xc5\r\x17\xd4\xba\xef-\xbc\xffKN\x12@mG\xcf\x1c\xe7b\x86\x8b\x9e\x8a$_H~\x82\x8e\xb2\x83\x87\xfe\x10H1L\x85\xa3n\x8e$\x0ej\xa5|\xe9\xdd5\xb4$\xee\x05\xd2\x8e\xc0\xbb\xc9\x0f\xa5\xf4U\x02\x96\xbf\xa4g\x02\xb1b\xa9\x06\nbmo9\xc9\xa4,\xef\xd1\x1fE\xd1m\xf4\'\x12\x9b\x19\xff\xb2v\xc2\x00\xdc{x\x97\xdd\xa3\x870,\xdd\x05\xc3\x12\xff1z5W\xf0\x9c\xe9\x8a\x82\x08:\xd7Mt\xbc.?D_\x17\x19\x980\x17\xf7\x90B\xd9\xcf\xe9`\x0cN\xebc.3p]4Ma\xe83l\xb6\x81\xd2\xe7\xb1\x94C\x84.KH\x05\xc55\xc8q\xc0R\xbc\x9cn$E\xe8\xf8T\x11\x93B\xfc\xd9\x93\xb2S)\xda\x9f\xd0Y\x88\xcbJUHYo\x93\xf4\xe0\xd6\xb5\xa5\xfbRN\xb6\xf4P\xfcl\x899L\xe5\xf8vSblD\xd9\x93X\xfcM\xa0+\x16\xe9BuNOk^\nB\x8c\xb6Y\xfd\x90\xccFu3\x109*GVy\xae\rR\x0eg\x8f\x19\xba\x98\xf4\xe4:\xe1\xf6!d\x86\x95\x11\xfeT\xe9\x08\xae\xe8l\xa4e9?\xa3o\x9a\x8fp\x9f\\HA\xc5y`\xb2R\\\x93i\xfa\x0b\x96@%\xe4\x04\xd1\xf4\xb4\xf9\x06\x9erf\x05rI\xcfa"b\x9f\xc0O:\xb8x\xe0RS\x1aTA\xaaZJ=\xd2QiPX\xe9hy\xc6\xca\x11$\x93m{\x17tH\x9f\x0c\xbf\x03?\xe9\xb0k\xa5bpk\xc6\x84t\x8e\x0e\x89\x8e\xd4\x87\xf1\x9c!\xbd\x06m2\xab\xfck:L\x8e\xeag:8S\x8c\x84\xb7\x99\xa58\xbb\\\xd0\xa3\xa3\xaf\xe1\x92>m[\xff\x81\xbe \xa6<\x1c\x89\x14\xc3\xb9\xa2\x97$=3\xfaK:\xbciw\x05\xbc\xd1\x9b\xb7\xcc\xaf\xf8\xd3\x1f\xd4\x92\xa2\xaf\xea\x82\x9e\x1f\xcc\xd5rI\xaf(\x85\xb6\x95n{S\xafC\xf0\xe0\x1b\x94\x93\xc0\\`\x1a\x99\xb3\xa2YM\x06\xa7tw/(!\r\xdd*\xf4\x84\x0e\x12\x8e\xadRP\xb3\xdd\xfb\x89\x0bU\x07g\x89\x160\x9fN\x9cN\xef)\xdd\t\xf6\xd6\xf5\xda\xe9j\xdd\x0b\xfdAg\xfb)\xc1\xc4\xc7\xc7;\xae#\rC\x0c\x8b;\xcf\xeeb\xbb\r\xe6\xe2\x94\x8e\xa9\x0f\x08\xcf\xbc\xc3yG\xbfA{J\xa7[\xac\xc1\xcd\xb1H\x1eU\x83\xbeG\x8a\xcc\xb3\xdd\x136\x1f\xe9\xe6{Ag\x18\xe1\xd8\x84\xdez\xecu\xd8\xb0 \x81y\x8a\xe1U\xec\x8dN\xdd1\xceC\x0f[\x99\xff~\xbfw\xfaFm\xb1u\xc81c\xd4\xd2\x05\xbf\xa4\xabd\x90\x87a\xe9\xaa%C\xc8\x99\xc0\xfeu\xa4+\xa7\xda\x96\xf7\xce\xc7\xd7Ecz"OSdbv\x84\xb6\x142`Z\\B\xe98\x17\xb22\x89\x8a\xdd\t\xcfUm4\xddQ\xd98\xb4.\xd21\x84\xb07\xfdF_S\xe2\xc9\xbe}\xdb`\x1f\x8f\x9f\xa4X\x96>TV\xe6\x89~\x9d\x96\xba\x92\x99\xa7u\xeaz\xc6#/\xf22j\x19|\x91\xa1\xd2\x9e\xb9Q{\x1b\x955ZA*\x13\xafx\xbf\xf1j\xb0\xa8\xd9\xd9\x97\xd5\x93"\x1bz\xbf\n\xb6\xa3\xf7\x07\xc3\x16a\xf6\xf7\x8f\xca\xbf\xa7C\xa6\x9e>\xd4+\x18\xca\x87\xfa\x07\xfa\x7f\x8c/\x01\x06\x00\xcc\x9eX\xf3u\xe2\x94\xf6\x00\x00\x00\x00IEND\xaeB`\x82',
        ),
        ],
    from_values=EmailAddress(name='Команда Почты Mail.Ru', email='welcome@corp.mail.ru'),
    to_values=(EmailAddress(name='', email='imap.tools@mail.ru'),),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(),
)


from typing import Any


class ImapToolsError(Exception):
    """Base lib error"""


class MailboxFolderStatusValueError(ImapToolsError):
    """Wrong folder status value error"""


class UnexpectedCommandStatusError(ImapToolsError):
    """Unexpected status in IMAP command response"""

    def __init__(self, command_result: tuple, expected: Any):
        """
        :param command_result: imap command result
        :param expected: expected command status
        """
        self.command_result = command_result
        self.expected = expected

    def __str__(self):
        return (f'Response status "{self.expected}" expected, '
                f'but "{self.command_result[0]}" received. Data: {str(self.command_result[1])}')


class MailboxFolderSelectError(UnexpectedCommandStatusError):
    pass


class MailboxFolderCreateError(UnexpectedCommandStatusError):
    pass


class MailboxFolderRenameError(UnexpectedCommandStatusError):
    pass


class MailboxFolderDeleteError(UnexpectedCommandStatusError):
    pass


class MailboxFolderStatusError(UnexpectedCommandStatusError):
    pass


class MailboxFolderSubscribeError(UnexpectedCommandStatusError):
    pass


class MailboxLoginError(UnexpectedCommandStatusError):
    pass


class MailboxLogoutError(UnexpectedCommandStatusError):
    pass


class MailboxNumbersError(UnexpectedCommandStatusError):
    pass


class MailboxUidsError(UnexpectedCommandStatusError):
    pass


class MailboxStarttlsError(UnexpectedCommandStatusError):
    pass


class MailboxFetchError(UnexpectedCommandStatusError):
    pass


class MailboxExpungeError(UnexpectedCommandStatusError):
    pass


class MailboxDeleteError(UnexpectedCommandStatusError):
    pass


class MailboxCopyError(UnexpectedCommandStatusError):
    pass


class MailboxMoveError(UnexpectedCommandStatusError):
    pass


class MailboxFlagError(UnexpectedCommandStatusError):
    pass


class MailboxAppendError(UnexpectedCommandStatusError):
    pass


class MailboxTaggedResponseError(UnexpectedCommandStatusError):
    pass



import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='Microsoft Vlsta & Office2007, Just released for 79$ Save 1599.95$ 0ff Retai|',
    from_='subventive@vodtravel.com',
    to=('sydney@xxxxxxxx.xxx',),
    cc=(),
    bcc=(),
    reply_to=(),
    date=datetime.datetime(1900, 1, 1, 0, 0),
    date_str='Pn, 29 paX 2007 21:13:00 +0100',
    text='hotnewsoft . com\r\n\r\n\r\n\r\n\r\n',
    html='',
    headers={'return-path': ('<me@xxx.xxxxxxxx.xxx.xx>',), 'x-original-to': ('external@xxx.xxxxxxxx.xxx.xx',), 'delivered-to': ('external@xxx.xxxxxxxx.xxx.xx',), 'received': ('from server.xxx.xxxxxxxx.xxx.xx (localhost.xxxxxxxx.xxx.xx [127.0.0.1])\r\n        by server.xxx.xxxxxxxx.xxx.xx (Postfix) with ESMTP id 0173828456\r\n        for <external@xxx.xxxxxxxx.xxx.xx>; Mon,  5 Nov 2007 20:17:37 +1100 (EST)', 'from server.xxx.xxxxxxxx.xxx.xx (server.xxx.xxxxxxxx.xxx.xx [10.130.1.250])\r\n        by localhost (FormatMessage) with SMTP id d7be4d19ed6d330c\r\n        for <sydney@xxxxxxxx.xxx>; Mon, 05 Nov 2007 20:17:37 +1100 (EST)', 'from server.sydney.xxxxxxxx.xxx.xx (unknown [10.130.2.6])\r\n        by server.xxx.xxxxxxxx.xxx.xx (Postfix) with ESMTP id BB00328442\r\n        for <sydney@xxxxxxxx.xxx>; Mon,  5 Nov 2007 20:17:37 +1100 (EST)', 'from dircomm (unknown [10.130.2.8])\r\n        by server.sydney.xxxxxxxx.xxx.xx (Postfix) with ESMTP id 20A692841F\r\n        for <sydney@xxxxxxxx.xxx>; Mon,  5 Nov 2007 20:18:24 +1100 (EST)'), 'x-mailbox-line': ('From subventive@vodtravel.com',), 'message-id': ('<000001c81a67$a4450700$0100007f@localhost>',), 'from': ('"Darrell Shaw" <subventive@vodtravel.com>',), 'to': ('<sydney@xxxxxxxx.xxx>',), 'subject': ('Microsoft Vlsta & Office2007, Just released for 79$ Save 1599.95$ 0ff Retai|',), 'date': ('Pn, 29 paX 2007 21:13:00 +0100',), 'content-type': ('text/plain;\r\n    charset="us-ascii"',), 'content-transfer-encoding': ('7bit',), 'x-priority': ('3 (Normal)',), 'x-msmail-priority': ('Normal',), 'x-mailer': ('Microsoft Outlook, Build 10.0.3416',), 'importance': ('Normal',), 'x-mimeole': ('Produced By Microsoft MimeOLE V6.00.2800.150',), 'x-csi-tmailscanner-information': ('Please contact the ISP for more information',), 'x-csi-tmailscanner': ('Found to be clean',), 'x-csi-tmailscanner-spamcheck': ('not spam, SpamAssassin (score=1.87, required 4,\r\n        autolearn=disabled, INVALID_DATE 0.23, RAZOR2_CF_RANGE_51_100 1.49,\r\n        RAZOR2_CHECK 0.15)',), 'x-csi-tmailscanner-spamscore': ('s',), 'x-tmailscanner-from': ('subventive@vodtravel.com',), 'resent-date': ('Mon, 05 Nov 2007 20:17:37 +1100 (EST)',), 'resent-from': ('<mail_dump@ns1.xxxxxxxx.xxx.xx>',), 'resent-to': ('<mikel@xxxxxxxx.xxx>',), 'resent-message-id': ('<d7be4d19ed6d330c.1194254257@xxxxxxxx.xxx.xx>',), 'x-uid': ('4702',), 'status': ('RO',)},
    attachments=[],
    from_values=EmailAddress(name='Darrell Shaw', email='subventive@vodtravel.com'),
    to_values=(EmailAddress(name='', email='sydney@xxxxxxxx.xxx'),),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(),
)


import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='Testmail',
    from_='jorn@prikkprikkprikk.no',
    to=('aftest@adfontesmedier.no',),
    cc=(),
    bcc=(),
    reply_to=(),
    date=datetime.datetime(2009, 10, 30, 20, 7, 42, tzinfo=datetime.timezone(datetime.timedelta(seconds=3600))),
    date_str='Fri, 30 Oct 2009 20:07:42 +0100',
    text='\r\n\r\n--\r\nJørn Støylen -- http://www.prikkprikkprikk.no\r\n924 38 051 -- jorn@prikkprikkprikk.no\r\n\r\n',
    html='',
    headers={'received': ('from mx01.medieveven.no (192.168.42.42) by epost.vartland.no\r\n (192.168.73.7) with Microsoft SMTP Server id 8.1.393.1; Fri, 30 Oct 2009\r\n 20:08:07 +0100', 'from woodward.joyent.us ([8.12.42.230])  by mx01.medieveven.no with\r\n ESMTP; 30 Oct 2009 20:11:23 +0100', 'from [10.0.0.5] (ti0083a340-0282.bb.online.no [88.89.59.28])\tby\r\n woodward.joyent.us (Postfix) with ESMTPSA id EC35240C85\tfor\r\n <aftest@adfontesmedier.no>; Fri, 30 Oct 2009 19:07:44 +0000 (GMT)'), 'from': ('=?iso-8859-1?Q?J=F8rn_St=F8ylen?= <jorn@prikkprikkprikk.no>',), 'to': ('AF Test <aftest@adfontesmedier.no>',), 'date': ('Fri, 30 Oct 2009 20:07:42 +0100',), 'subject': ('Testmail',), 'thread-topic': ('Testmail',), 'thread-index': ('AcpZlFLF/Y9EfcC0QZKKEuUFm2Snqw==',), 'message-id': ('<4FDC124A-1FC4-4B29-8502-E459BD9AB397@prikkprikkprikk.no>',), 'accept-language': ('nb-NO',), 'x-ms-exchange-organization-authas': ('Anonymous',), 'x-ms-exchange-organization-authsource': ('epost.vartland.no',), 'x-ms-has-attach': ('',), 'x-ms-tnef-correlator': ('',), 'x-ironport-anti-spam-filtered': ('true',), 'x-ironport-av': ('E=Sophos;i="4.44,655,1249250400";    d="scan\'208";a="2439461"',), 'x-ironport-anti-spam-result': ('AvMBAPfV6koIDCrmhWdsb2JhbACbVgEBAQoLChoDxH2EPQQ',), 'content-type': ('text/plain; charset="iso-8859-1"',), 'content-transfer-encoding': ('quoted-printable',), 'mime-version': ('1.0',)},
    attachments=[],
    from_values=EmailAddress(name='Jørn Støylen', email='jorn@prikkprikkprikk.no'),
    to_values=(EmailAddress(name='AF Test', email='aftest@adfontesmedier.no'),),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(),
)


import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='',
    from_='anonymous@i.tp.host',
    to=('user-example@aol.com', 'e-s-a-s-2200@app.ar.com'),
    cc=(),
    bcc=(),
    reply_to=(),
    date=datetime.datetime(2010, 10, 14, 23, 25, 6, tzinfo=datetime.timezone(datetime.timedelta(days=-1, seconds=72000))),
    date_str='14 Oct 2010 23:25:06 -0400',
    text='\r\nCONTACT:\r\n\r\n\r\n\r\n\r\nCOMMENT:\r\n\r\n\r\nPAGE THEY WERE ON:\r\n\r\n',
    html='',
    headers={'delivered-to': ('e-s-a-s-2200@app.ar.com',), 'received': ('by 10.103.1.1 with SMTP id w10cs50835muo;\r\n        Thu, 14 Oct 2010 20:25:16 -0700 (PDT)', 'by 10.150.205.4 with SMTP id c4mr736200ybg.26.1287113115711;\r\n        Thu, 14 Oct 2010 20:25:15 -0700 (PDT)', 'from i.tp.host ([172.1.1.1])\r\n        by mx.google.com with ESMTP id s21si2123456.90.2010.10.14.20.25.15;\r\n        Thu, 14 Oct 2010 20:25:15 -0700 (PDT)', '(qmail 28454 invoked by uid 48); 14 Oct 2010 23:25:06 -0400'), 'return-path': ('<anonymous@i.tp.host>',), 'received-spf': ('neutral (google.com: 172.1.1.1 is neither permitted nor denied by best guess record for domain of anonymous@i.tp.host) client-ip=172.1.1.1;',), 'authentication-results': ('mx.google.com; spf=neutral (google.com: 172.1.1.1 is neither permitted nor denied by best guess record for domain of anonymous@i.tp.host) smtp.mail=anonymous@i.tp.host',), 'date': ('14 Oct 2010 23:25:06 -0400',), 'message-id': ('<20101015032506.28448.qmail@i.tp.host>',), 'from': ('anonymous@i.tp.host',), 'to': (', user-example@aol.com, e-s-a-s-2200@app.ar.com',), 'subject': ('',)},
    attachments=[],
    from_values=EmailAddress(name='', email='anonymous@i.tp.host'),
    to_values=(EmailAddress(name='', email='user-example@aol.com'), EmailAddress(name='', email='e-s-a-s-2200@app.ar.com')),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(),
)


"""How to use keyword criteria"""

from imap_tools import A, MailBox

with MailBox('imap.wow.ru').xoauth2('imap.tools', '123') as mailbox:
    uids = ['183']
    # get messages by uid
    for msg in mailbox.fetch(A(uid=uids)):
        print(msg.uid, msg.subject)
    # add custom flag
    mailbox.flag(uids, ['flag1'], True)
    # get messages by custom flag
    for msg in mailbox.fetch(A(keyword='flag1')):
        print(msg.uid, msg.subject, msg.flags)



import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='this message JUST contains an attachment',
    from_='rfinnie@domain.dom',
    to=('bob@domain.dom',),
    cc=(),
    bcc=(),
    reply_to=(),
    date=datetime.datetime(2003, 10, 23, 22, 40, 49, tzinfo=datetime.timezone(datetime.timedelta(days=-1, seconds=61200))),
    date_str='23 Oct 2003 22:40:49 -0700',
    text='',
    html='',
    headers={'subject': ('this message JUST contains an attachment',), 'from': ('Ryan Finnie <rfinnie@domain.dom>',), 'to': ('bob@domain.dom',), 'content-disposition': ('attachment; filename=blah.gz',), 'content-transfer-encoding': ('base64',), 'content-description': ('Attachment has identical content to above foo.gz',), 'message-id': ('<1066974048.4264.62.camel@localhost>',), 'mime-version': ('1.0',), 'date': ('23 Oct 2003 22:40:49 -0700',), 'content-type': ('application/x-gzip; NAME=blah.gz',)},
    attachments=[
        dict(
            filename='blah.gz',
            content_id='',
            content_disposition='attachment',
            content_type='application/x-gzip',
            payload=b'J\xe6\xe3y\xcbm\x86+&z\xcb\x1a\x81\xe2TI7(\x9e\xd6\xa2\x9e\xc6\xa7j\xdbZr\x19\x9e\x9e\xd1k\xa2drjqb\x9ex\x9e\xad\xf8\xa7\x9e\'\x9d\xa2f\xa2\x9d\xda&N\x86\xe8m\xda&j)\xdd\xa2`\xa8\x9e\xd7\xa7\xb48\xac\xa6\x8b"\xb6*\'j\xdbZr\x19\x9e\x9e\xd7\xe2\x95\xe9\xda\x99\xe0\x1b\x95\xa8`\xcc*\'\xb5\xe9\xedO*^j\x9ae\x89\xc6\xad\x8a\x89\xff\xc6\x0c\xe2\xa6v\xa6x\x06\xe5j\x183\n\x89\xedz{S\xad\xa9\xec}\xea\xc4\x9d\xca\x1d\x8ax\x1bj\xc7\xba\xe0*\'\xb5\xe9\xed\r\xeb\x1c\xae*m\x8a\x89\xc0\xb6\xd6\x9c\x86g\xa7\xb6\x16\xac\x89\xd7\xa7\xb6\'\x1a\x95\xca\'\xb5\xe9\xed\xb6\x86\x9b\xa2\xf7\x9f\xa2\x8831\xeb,j\x07\x88w]:\xeb\xde\xf8\xd3\x8f8\xdb\xae:\xd9\xc6\xa6zYhq\xa9a\xa2\xcbL\x8ag\x95z\xbb"\xa2}t\r\xab^\xdbs\x9c\xb7m4\xdfm\xb8\xd3\x8ft\xefM\x07\xe2\xc2\x008pf\x0f\xc0\x00\xe2\xf3\x0b\xd5s\xf2Tr\x12\xcbR\xcbT\x922\xf3\x12\xd4r\x13Uy\x00\x80\x1d\xb7\xbb\xeb\x85\x80',
        ),
        ],
    from_values=EmailAddress(name='Ryan Finnie', email='rfinnie@domain.dom'),
    to_values=(EmailAddress(name='', email='bob@domain.dom'),),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(),
)


from imap_tools import MailBox, AND

"""
Get date, subject and body len of all emails from INBOX folder

1. MailBox()
    Create IMAP client, the socket is created here
    
2. mailbox.login()
    Login to mailbox account
    It supports context manager, so you do not need to call logout() in this example
    Select INBOX folder, cause login initial_folder arg = 'INBOX' by default (set folder may be disabled with None)
    
3. mailbox.fetch()
    First searches email uids by criteria in current folder, then fetch and yields MailMessage
    Criteria arg is 'ALL' by default
    Current folder is 'INBOX' (set on login), by default it is INBOX too.
    Fetch each message separately per N commands, cause bulk arg = False by default
    Mark each fetched email as seen, cause fetch mark_seen arg = True by default
    
4. print
    msg variable is MailMessage instance
    msg.date - email data, converted to datetime.date
    msg.subject - email subject, utf8 str
    msg.text - email plain text content, utf8 str
    msg.html - email html content, utf8 str
"""
with MailBox('imap.mail.com').login('test@mail.com', 'pwd') as mailbox:
    for msg in mailbox.fetch():
        print(msg.date, msg.subject, len(msg.text or msg.html))

# Equivalent verbose version:
mailbox = MailBox('imap.mail.com')
mailbox.login('test@mail.com', 'pwd', 'INBOX')  # or use mailbox.folder.set instead initial_folder arg
for msg in mailbox.fetch(AND(all=True)):
    print(msg.date, msg.subject, len(msg.text or msg.html))
mailbox.logout()

"""
Mailbox class choosing:
    MailBox
        Corresponds to imaplib.IMAP4_SSL.
        Encrypted connection with SSL/TLS. This is what most email servers use these days.
    MailBoxTls
        Corresponds to imaplib.IMAP4, then using startls() on the resulting connection.
        For a STARTTLS connection: 
        creates a plaintext connection then upgrades it later by using a STARTTLS command in the protocol. 
        The internet has mostly gone to the "always encrypted" rather than "upgrade" paradigm.
    MailBoxUnencrypted
        Corresponds to imaplib.IMAP4 with no security applied.
        Standard IMAP without SSL/TLS. You should not use this on the public internet.
"""



import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='GA.comô has a lead for you',
    from_='j@yahoo-example.com',
    to=('c@ra-example.com', 'e-r-w-a-4462@app.ar.com', 'leads@ga-example.com'),
    cc=(),
    bcc=(),
    reply_to=(),
    date=datetime.datetime(2010, 10, 12, 16, 21, 5),
    date_str='Tue, 12 Oct 2010 16:21:05 H0500',
    text='Body Text - not important',
    html='',
    headers={'delivered-to': ('e-r-w-a-4462@app.ar.com',), 'received': ('by 10.1.1.1 with SMTP id w10cs10896muo;\r\n        Tue, 12 Oct 2010 13:20:24 -0700 (PDT)', 'by 10.2.1.5 with SMTP id bk15mr6105827qab.89.1286914824124;\r\n        Tue, 12 Oct 2010 13:20:24 -0700 (PDT)', 'from mail.ga-example.com (mail.ga-example.com [64.1.2.3])\r\n        by mx.google.com with ESMTP id 13si112341247qcd.23.2010.10.12.13.20.23;\r\n        Tue, 12 Oct 2010 13:20:24 -0700 (PDT)', 'from GAWWW03 by ga-example.com (MDaemon PRO v11.0.3)\r\n\twith ESMTP id md50004804310.msg\r\n\tfor <e-r-w-a-4462@app.ar.com>; Tue, 12 Oct 2010 16:21:05 -0400'), 'return-path': ('<j@yahoo-example.com>',), 'received-spf': ('neutral (google.com: 64.1.2.3 is neither permitted nor denied by best guess record for domain of j@yahoo-example.com) client-ip=64.1.2.3;',), 'authentication-results': ('mx.google.com; spf=neutral (google.com: 64.1.2.3 is neither permitted nor denied by best guess record for domain of j@yahoo-example.com) smtp.mail=j@yahoo-example.com',), 'x-mdav-processed': ('mail.ga-example.com, Tue, 12 Oct 2010 16:21:07 -0400',), 'x-spam-processed': ('mail.ga-example.com, Tue, 12 Oct 2010 16:21:06 -0400',), 'x-spam-checker-version': ('SpamAssassin 3.2.5 (2008-06-10) on\r\n\tMAIL02.VD-example.com',), 'x-spam-level': ('',), 'x-spam-status': ('No, score=-1.1 required=5.0 tests=BAYES_00,DATE_IN_PAST_03_06,\r\n\tFORGED_YAHOO_RCVD,INVALID_DATE,NO_RELAYS,SUBJECT_NEEDS_ENCODING\r\n\tshortcircuit=no autolearn=no version=3.2.5',), 'x-mdremoteip': ('192.168.254.73',), 'x-return-path': ('j@yahoo-example.com',), 'x-envelope-from': ('j@yahoo-example.com',), 'x-mdaemon-deliver-to': ('e-r-w-a-4462@app.ar.com',), 'date': ('Tue, 12 Oct 2010 16:21:05 H0500',), 'subject': ('GA.com\udcc3\udcb4 has a lead for you',), 'to': ('c@ra-example.com,e-r-w-a-4462@app.ar.com,leads@ga-example.com',), 'mime-version': ('1.0',), 'content-type': ('text/plain; charset=iso-8859-1',), 'from': ('j@yahoo-example.com',), 'message-id': ('<MDAEMON-F201010121621.AA2105420md50000198258@ga-example.com>',)},
    attachments=[],
    from_values=EmailAddress(name='', email='j@yahoo-example.com'),
    to_values=(EmailAddress(name='', email='c@ra-example.com'), EmailAddress(name='', email='e-r-w-a-4462@app.ar.com'), EmailAddress(name='', email='leads@ga-example.com')),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(),
)


import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='',
    from_='pete@silly.test',
    to=('c@public.example', 'joe@example.org', 'jdoe@one.test'),
    cc=(),
    bcc=(),
    reply_to=(),
    date=datetime.datetime(1969, 2, 13, 23, 32, tzinfo=datetime.timezone(datetime.timedelta(days=-1, seconds=73800))),
    date_str='Thu,\r\n      13\r\n        Feb\r\n          1969\r\n      23:32\r\n               -0330 (Newfoundland Time)',
    text='Testing.\r\n',
    html='',
    headers={'from': ('Pete(A wonderful \\) chap) <pete(his account)@silly.test(his host)>',), 'to': ("A Group(Some people)\r\n     :Chris Jones <c@(Chris's host.)public.example>,\r\n         joe@example.org,\r\n  John <jdoe@one.test> (my dear friend); (the end of the group)",), 'cc': ('(Empty list)(start)Undisclosed recipients  :(nobody(that I know))  ;',), 'date': ('Thu,\r\n      13\r\n        Feb\r\n          1969\r\n      23:32\r\n               -0330 (Newfoundland Time)',), 'message-id': ('<testabcd.1234@silly.test>',)},
    attachments=[],
    from_values=EmailAddress(name='Pete (A wonderful ) chap his account his host)', email='pete@silly.test'),
    to_values=(EmailAddress(name="Chris Jones (Chris's host.)", email='c@public.example'), EmailAddress(name='', email='joe@example.org'), EmailAddress(name='John', email='jdoe@one.test')),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(),
)


import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='Mail System Error - Returned Mail',
    from_='Postmaster@ci.com',
    to=('notification+promo@blah.com',),
    cc=(),
    bcc=(),
    reply_to=('Postmaster@ci.com',),
    date=datetime.datetime(2010, 6, 29, 10, 42, 44, tzinfo=datetime.timezone(datetime.timedelta(days=-1, seconds=68400))),
    date_str='Tue, 29 Jun 2010 10:42:44 -0500',
    text='This Message was undeliverable due to the following reason:\r\n<u@ci.com> has restricted SMS e-mail\r\nPlease reply to <Postmaster@ci.com>\r\nif you feel this message to be in error.Hey cingularmefarida,\n\nFarida Malik thinks you should apply to join HomeRun, your place fot., San Francisco, CA, 94123, USA',
    html="<!DOCTYPE html>\n<html>\n<head>\n<title>HomeRun - Your Friend Farida Malik wants you to join run.com/o.45b0d380.gif' width='1' />\n</td>\n</tr>\n</table>\n</td>\n</tr>\n</table>\n</div>\n</body>\n</html>\n",
    headers={'return-path': ('<>',), 'x-original-to': ('notification+promo@blah.com',), 'delivered-to': ('notification+promo@blah.com',), 'received': ('from schemailmta04.ci.com (schemailmta04.ci.com [209.183.37.58])\r\n    by blah.com (Postfix) with ESMTP id 24EF419F546\r\n    for <notification+promo@blah.com>; Tue, 29 Jun 2010 15:42:46 +0000 (UTC)',), 'to': ('notification+promo@blah.com',), 'from': ('Mail Administrator <Postmaster@ci.com>',), 'reply-to': ('<Postmaster@ci.com>',), 'subject': ('Mail System Error - Returned Mail',), 'date': ('Tue, 29 Jun 2010 10:42:44 -0500',), 'message-id': ('<20100629154244.OZPA15102.schemailmta04.ci.com@schemailmta04>',), 'mime-version': ('1.0',), 'content-type': ('multipart/report;\r\n        report-type=delivery-status;\r\n        Boundary="===========================_ _= 6078796(15102)1277826164"',), 'x-cloudmark-analysis': ('v=1.0 c=1 a=q8OS1GolVHwA:10 a=ev1gGZlfZ-EA:10 a=HQ-Cukr2AAAA:8 a=qihIh-XuXL65y3o_mUgA:9 a=mUL5bUDOV_-gjcCZylcY5Lz4jjsA:4 a=iQvSWfByulMA:10 a=ni8l3qMSI1sA:10 a=WHDNLAQ519cA:10 a=Fry9e7MVxuJdODrS104A:9 a=JYo4OF_E9TqbHrUN2TvLdggtx2cA:4 a=S0jCPnXDAAAA:8 a=pXkHMj1YAAAA:8 a=5ErcFzC0N3E7OloTRA8A:9 a=cC0RL7HlXt3RrKfnpEbxHCeM-zQA:4 a=cHEBK1Z0Lu8A:10 a=p9ZeupWRHUwA:10 a=7sPVfr_AX1EA:10',)},
    attachments=[
        dict(
            filename='',
            content_id='',
            content_disposition='',
            content_type='message/rfc822',
            payload=b'Received: from schemailedgegx04.ci.com ([172.16.130.170])\n          by schemailmta04.ci.com\n          (InterMail vM.6.01.04.00 201-2131-118-20041027) with ESMTP\n          id\n <20100629154237.OZBY15102.schemailmta04.ci.com@schemailedgegx04.ci.com>\n          for <u@ci.com>; Tue, 29 Jun 2010 10:42:37 -0500\nReceived: from blah.com ([1.1.1.1])\n          by schemailedgegx04.ci.com\n          (InterMail vG.1.02.00.04 201-2136-104-104-20050323) with ESMTP\n          id <20100629154225.WEFB17009.schemailedgegx04.ci.com@blah.com>\n          for <u@ci.com>; Tue, 29 Jun 2010 10:42:25 -0500\nReceived: from blah.com (snooki [10.12.126.68])\n    by blah.com (Postfix) with ESMTP id 4BDAE19F546\n    for <u@ci.com>; Tue, 29 Jun 2010 15:42:25 +0000 (UTC)\nDKIM-Signature: v=1; a=rsa-sha256; c=simple/simple; d=blah.com;\n    s=2010; t=1277826145; bh=wC3hHAhQgApcTmwQsi2F4OJf40rbyIek/WwIuzSc3V\n    M=; h=Date:From:Reply-To:To:Message-ID:Subject:Mime-Version:\n     Content-Type:Content-Transfer-Encoding:List-Unsubscribe; b=aw+Bhd8\n    t1goZUXWBAHSrHaM1IdqhkXqF5WVMwGRYcnya4FHNw05XfpB3TTpTFda13DfhtziFRk\n    zHSfiNbMapv7Vz+D3A/9NHg5nKahSMosZVTa0BfajYWNd1aY8JUWUlxdQHxQQ4ygCBj\n    /MndJohtSm6K3gsqdIv88DNXdBGBEw=\nDate: Tue, 29 Jun 2010 15:42:25 +0000\nFrom: HomeRun <notification@blah.com>\nReply-To: HomeRun <notification+45b0d380@blah.com>\nTo: u@ci.com\nMessage-ID: <4c2a146147ac8_61ff157c4ec1652df@s.h.c.mail>\nSubject: Your Friend F M wants you to join HomeRun\nMime-Version: 1.0\nContent-Type: multipart/alternative;\n boundary="--==_mimepart_4c2a146141756_61ff157c4ec1649a8";\n charset=UTF-8\nContent-Transfer-Encoding: 7bit\nList-Unsubscribe: <mailto:unsubscribe+45b0d380@blah.com>\n\n\n\n----==_mimepart_4c2a146141756_61ff157c4ec1649a8\nDate: Tue, 29 Jun 2010 15:42:25 +0000\nMime-Version: 1.0\nContent-Type: text/plain;\n charset=UTF-8\nContent-Transfer-Encoding: base64\nContent-ID: <4c2a146145451_61ff157c4ec165040@s.h.c.mail>\n\nSGV5IGNpbmd1bGFybWVmYXJpZGEsCgpGYXJpZGEgTWFsaWsgdGhpbmtzIHlv\ndSBzaG91bGQgYXBwbHkgdG8gam9pbiBIb21lUnVuLCB5b3VyIHBsYWNlIGZv\ndC4sIFNhbiBGcmFuY2lzY28sIENBLCA5NDEyMywgVVNB\n\n\n----==_mimepart_4c2a146141756_61ff157c4ec1649a8\nDate: Tue, 29 Jun 2010 15:42:25 +0000\nMime-Version: 1.0\nContent-Type: text/html;\n charset=UTF-8\nContent-Transfer-Encoding: base64\nContent-ID: <4c2a1461468ae_61ff157c4ec165194@s.h.c.mail>\n\nPCFET0NUWVBFIGh0bWw+CjxodG1sPgo8aGVhZD4KPHRpdGxlPkhvbWVSdW4g\nLSBZb3VyIEZyaWVuZCBGYXJpZGEgTWFsaWsgd2FudHMgeW91IHRvIGpvaW4g\ncnVuLmNvbS9vLjQ1YjBkMzgwLmdpZicgd2lkdGg9JzEnIC8+CjwvdGQ+Cjwv\ndHI+CjwvdGFibGU+CjwvdGQ+CjwvdHI+CjwvdGFibGU+CjwvZGl2Pgo8L2Jv\nZHk+CjwvaHRtbD4K\n\n\n----==_mimepart_4c2a146141756_61ff157c4ec1649a8--\n',
        ),
        
        dict(
            filename='',
            content_id='4c2a146145451_61ff157c4ec165040@s.h.c.mail',
            content_disposition='',
            content_type='text/plain',
            payload=b'Hey cingularmefarida,\n\nFarida Malik thinks you should apply to join HomeRun, your place fot., San Francisco, CA, 94123, USA',
        ),
        
        dict(
            filename='',
            content_id='4c2a1461468ae_61ff157c4ec165194@s.h.c.mail',
            content_disposition='',
            content_type='text/html',
            payload=b"<!DOCTYPE html>\n<html>\n<head>\n<title>HomeRun - Your Friend Farida Malik wants you to join run.com/o.45b0d380.gif' width='1' />\n</td>\n</tr>\n</table>\n</td>\n</tr>\n</table>\n</div>\n</body>\n</html>\n",
        ),
        ],
    from_values=EmailAddress(name='Mail Administrator', email='Postmaster@ci.com'),
    to_values=(EmailAddress(name='', email='notification+promo@blah.com'),),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(EmailAddress(name='', email='Postmaster@ci.com'),),
)


import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='[Online Lead] Online Lead #1111111',
    from_='l@gcn-example.com',
    to=('leads@sg.dc.com', 'sag@leads.gs.ry.com', 'sn@example-hotmail.com', 'e-s-a-g-8718@app.ar.com', 'jp@t-exmaple.com', 'cc@c-l-example.com'),
    cc=(),
    bcc=(),
    reply_to=(),
    date=datetime.datetime(2010, 10, 13, 7, 53, 4, tzinfo=datetime.timezone(datetime.timedelta(days=-1, seconds=61200))),
    date_str='Wed, 13 Oct 2010 07:53:04 -0700',
    text='<?xml version="1.0" encoding="UTF-8"?>\r\n<?ADF version="1.0"?><adf>\r\n  <prospect>\r\n    <requestdate>2010-10-13T07:53:04-09:00</requestdate>\r\n    ... (not important) ...',
    html='',
    headers={'delivered-to': ('e-s-a-g-8718@app.ar.com',), 'received': ('by 10.2.1.1 with SMTP id c60cs12954wel;\r\n        Wed, 13 Oct 2010 07:44:39 -0700 (PDT)', 'by 10.15.1.1 with SMTP id f12mr1139775ybe.360.1286981078364;\r\n        Wed, 13 Oct 2010 07:44:38 -0700 (PDT)', 'from services.travidiabamboo.com (services.travidiabamboo.com [72.21.1.1])\r\n        by mx.google.com with ESMTP id v33si16714378yba.88.2010.10.13.07.44.37;\r\n        Wed, 13 Oct 2010 07:44:38 -0700 (PDT)', 'from travidiabamboo.com (services [10.0.0.2])\r\n\tby services.travidiabamboo.com (Postfix) with ESMTP id A532170702;\r\n\tWed, 13 Oct 2010 07:53:04 -0700 (PDT)'), 'return-path': ('<l@gcn-example.com>',), 'received-spf': ('pass (google.com: domain of l@gcn-example.com designates 72.21.1.1 as permitted sender) client-ip=72.21.1.1;',), 'authentication-results': ('mx.google.com; spf=pass (google.com: domain of l@gcn-example.com designates 72.21.1.1 as permitted sender) smtp.mail=l@gcn-example.com',), 'date': ('Wed, 13 Oct 2010 07:53:04 -0700',), 'from': ('l@gcn-example.com',), 'to': ('leads@sg.dc.com,\r\n\t sag@leads.gs.ry.com,\r\n\t sn@example-hotmail.com,\r\n\t e-s-a-g-8718@app.ar.com,\r\n\t jp@t-exmaple.com,\r\n\t\r\n\tcc@c-l-example.com',), 'message-id': ('<4cb5c7d0a3cce_120e..fdbed2b861958562@s.t-example.com.tmail>',), 'subject': ('[Online Lead] Online Lead #1111111',), 'mime-version': ('1.0',), 'content-type': ('text/plain; charset=utf-8',)},
    attachments=[],
    from_values=EmailAddress(name='', email='l@gcn-example.com'),
    to_values=(EmailAddress(name='', email='leads@sg.dc.com'), EmailAddress(name='', email='sag@leads.gs.ry.com'), EmailAddress(name='', email='sn@example-hotmail.com'), EmailAddress(name='', email='e-s-a-g-8718@app.ar.com'), EmailAddress(name='', email='jp@t-exmaple.com'), EmailAddress(name='', email='cc@c-l-example.com')),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(),
)


import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='test',
    from_='samgentlemankaka@foxmail.com',
    to=('qmebi@mfk.app',),
    cc=(),
    bcc=(),
    reply_to=(),
    date=datetime.datetime(2020, 10, 21, 11, 34, 9, tzinfo=datetime.timezone(datetime.timedelta(seconds=28800))),
    date_str='Wed, 21 Oct 2020 11:34:09 +0800',
    text='',
    html='<meta http-equiv="Content-Type" content="text/html; charset=utf-8"><div><img src="cid:F76361FC@E3C5215F.31AC8F5F.png" style="" id="img_insert_160325124624005018289219499459"><br><br></div>',
    headers={'return-path': ('<samgentlemankaka@foxmail.com>',), 'x-original-to': ('qmebi@mfk.app',), 'delivered-to': ('qmebi@mfk.app',), 'received': ('from qq.com (unknown [113.96.223.80])\r\n\tby mail.mfk.app (Postfix) with ESMTP id DD4B62140DD4\r\n\tfor <qmebi@mfk.app>; Wed, 21 Oct 2020 03:34:11 +0000 (UTC)', 'from qq.com (unknown [127.0.0.1])\r\n\tby smtp.qq.com (ESMTP) with SMTP\r\n\tid ; Wed, 21 Oct 2020 11:34:10 +0800 (CST)'), 'dkim-signature': ('v=1; a=rsa-sha256; c=relaxed/relaxed; d=foxmail.com;\r\n\ts=s201512; t=1603251251;\r\n\tbh=bZSz/mtqfdz94kmtGaFYXC+YAQiOxaGFSWBJgZKsdU4=;\r\n\th=From:To:Subject:Mime-Version:Date:Message-ID;\r\n\tb=bHrL4mJuop2NculsqdAgtZBTFjvA8VK2WqfJwn4MqmVpNUvfUQ/OyCzC7WAG2DjDq\r\n\t NhaAt5+61RCFDBKuv2NVx7xyH7xPTI0xVrCbUZinXDQQ53r6vBwP08jwx4GaP+b4k6\r\n\t WYypqGbGKA2pDDUKmsABkHWdG2QbANM0YDCLnlBM=',), 'x-qq-feat': ('AU3gs7VM8fUm1shbPQ2y/acBleNQwil+zNLsynIgszFw26Mnhh/EwwA+B+R36\r\n\tyfTKfZtupqkoghvYDKTdaoIJHPvl3hwvMYCK5O0pgoIgbwuFYw2V5fn3Gldt7UXsU6U7tyH\r\n\twqWfCV4fZ445hfnnbUWSDNVElNzvEyQ+KXbeWoY8wscD6YadaE8uZDImlxSshpPqQ6QZ+Hv\r\n\tLAH0CFTPcbU0pVX7JZWe+yehkkfGa8pNbg2udDcM+l2It8FiRxZE1Dd/e+eLR6ZeHxrSdcF\r\n\tzsYA==',), 'x-qq-ssf': ('00000000000000F100000000000000J',), 'x-qq-xmailinfo': ('MKhv4vGKTxZzZwBiOOWzJjOs1CMM2urx0EOWlCdHAK4tftFk+6UX7CdZHFmrau\r\n\t guT9pUXQHsD4SoKYAbJJgAkezWTcclYEtZ5xC4nTjAObSp7hATM9Wz70KG1uocKqaXQT7MJP0sc9c\r\n\t bpSK6QNPNd3+j7/BptWtYSJjEQSs7BFqvSDf3T7b1gPmf14cCPF4OGaIV332nu7Yfv6UeJDuMtM8E\r\n\t V8WKUfbWqjedOX7j/4mx94I9GBJK7MwebhIqqVST1wrdrAckfU+wzSRmn+GsvqRnYuog5vBEtUjMW\r\n\t dHVOpXPD3GtA0tqfs3mbYyV45ix3wR5P3AQQoVgyyFiIMVWW2Ro/pCCxW4t8IGJLDWtm7SE1x+n5l\r\n\t ovf0oGXFRVI14LDD6ANU8IspSEEhCNBy8VY8j4nAIbME2fbMvnlP65Ypa9e7HpLYcaJryAp/z0qh2\r\n\t qUVXkbM7NBGR/gQfTjUa9HduSGNfUAdrPy/PCFyUr6TzuKfWdv1oaHe5muNm91NtaCZdPcy+bTNYf\r\n\t oI6eyoMTJflL8zriZXN7PjVgtZBrRkuGLDBBphNYVLKnqW0Pf9UQf46pCuQvvneVPqSb3pSzoy+O2\r\n\t fiO8+pRyy/J/L9YrHExFlgz38D8S1qYzUq5nGEBOvkTiaJZfu/ie5Gmv0a8W0CkWEqmNgGCg556Wr\r\n\t cSs9YFvPuleEF6mSFSzbj7R1dz4Q==',), 'x-has-attach': ('no',), 'x-qq-business-origin': ('2',), 'x-originating-ip': ('112.80.214.47',), 'x-qq-style': ('',), 'x-qq-mid': ('webmail615t1603251249t9058947',), 'from': ('"=?ISO-8859-1?B?c2g=?=" <samgentlemankaka@foxmail.com>',), 'to': ('"=?ISO-8859-1?B?cW1lYmk=?=" <qmebi@mfk.app>',), 'subject': ('test',), 'mime-version': ('1.0',), 'content-type': ('multipart/related;\r\n\ttype="multipart/alternative";\r\n\tboundary="----=_NextPart_5F8FAC31_11789E80_23290B52"',), 'content-transfer-encoding': ('8Bit',), 'date': ('Wed, 21 Oct 2020 11:34:09 +0800',), 'x-priority': ('3',), 'message-id': ('<tencent_DF1A0C2499D7CA259C7FB26CDFCB49C60809@qq.com>',), 'x-qq-mime': ('TCMime 1.0 by Tencent',), 'x-mailer': ('QQMail 2.x',), 'x-qq-mailer': ('QQMail 2.x',), 'x-qq-sendsize': ('520',), 'feedback-id': ('webmail:foxmail.com:bgweb:bgweb17',)},
    attachments=[
        dict(
            filename='F76361FC@E3C5215F.31AC8F5F.png.jpg',
            content_id='F76361FC@E3C5215F.31AC8F5F.png',
            content_disposition='',
            content_type='image/jpeg',
            payload=b'\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00s\x00\x00\x001\x08\x06\x00\x00\x00\xe9\xd3\x96\x81\x00\x00\x03ZIDATx\x01\xed\x9bAh\x13A\x14\x86\xffhA\x0cT\xa4\xa9P\xc9A\x8a\x17%\x97\xf4\xe4!t!\xe4fo\x1e\x02\x01o\x81B x\xec)x(9\xe5$\xa5\x10\x08\xf4&\x88\x01{\xab\x07A\x8a\t9xR\x84\x90\x16\t\xa2RZ\xb0\xf1\xa0\x10\xf1 \x91\xd9\xec\xa6;\xb3\t\xd6N\xb0o\xc6\x17\x08\xc9\x9b\x9d\x9d\xbc\xf7\x7f\xf3\xde\xce\x0e\x9bH\xe5\xf9\xe1\x00\xfc\xb2B\x81\x0bVD\xc1A\xb8\n0L\x8b&\xc2\xcc\xfb\xee\x07\x8b\xc2\xf9\xbfC\x89\x0c\x06\x03\xbefZ2\x07\xb8\xccZ\x02R\x84\xc10\x19\xa6E\nX\x14\ng&\xc3\xb4H\x01\x8bB\xe1\xcc\xb4\x08\xe6\x8c\x1aK\xbf\xdf\xc7\x97\x1f\x17\xd5f\xb6\t)p\xed\xf2/D\xa3\xd1\x90G\xd2}\xe6\xc7\xde\xcfP\x07n\xa0\xab\xc0\x8d\xd8%\xc9\xb9Q\x99\x15\x19\xc9/\xb3\x14P\x99\x8d`ri5\x0b\xa4\xf0Ve6\x82i^(\xec\xb1\xaa\x00\xc3T\x151\xd8f\x98\x06\xc3S]g\x98\xaa"\x06\xdb\x0c\xd3`x\xaa\xeb\x0cSU\xc4`\x9ba\x1a\x0cOu=\xb4\x9d\xa7v\xf8\xe7v\xaf\x85\x87\xa56\x9a\xc1\x1fv\x1c\xec&\xbbHo\x1c\x04[\xbd\xef\xb3(\x95s\xc8\xc4\x86f\xfbI\r\xc5\x06\x90\x7f\xb0\x8a\xfb\xb7\xe5\xb6\xe5\xec=\xac\xa7\xe7\xc7\x8c\x01\x8c;Ot\x9c\xd4>v\x90sn\xa4\x9b\x99\x02`\xd5A^\x11H\x00\xd9\xad\xae\xba\xefMG9\x88=\xbci\x88\xb68\x96<\x90\xc2J$\xe3n\xc7f\xfd5\xda\xea)\x16\xd9\xf42SG\xdcN\x17[\xe2|\xe7*\x8e*5\x14C\xcf\xaa\x1d\xa0X\xa8\x9d\xfc\xc2b\x02\xcf\xd6R\x98;i1\xfa\x9bU0\xdbo\x87e8\x9fL!\x93K!s\x064[\x1b\xb5\xe1\x84P\xce\x95\xdaE\xd5\xc8\xddRz\x9c\xbfI\x17f\xa3\x81\xb4[2e\x91\x9a\xf5m\xa4\xebr\x9bk\xf5Zx\x1a\xec?\xee\xda+\x9d\x16\xc7fu\x05\t\xa9M\xbe\xd6*\x87\xc8\x9bta\xba\xb3\x1fx\\hH\x99\x12\\\xc4\xf8\x8b\x13\xa1r\xfb\x85\xb2h\x8a\xa5\xb0^M\x91\x070M\x07\xe9\xc2\xd4\x8d\xf2\x8c\x99)\x95S\xc9\x07y\xd5,\x1d"b\x18\x07sR\x99\xbd\xbe\x10G)\x0b\x94\xeb\xde\xed\xcb_e\xe61\x8e>\x0b"\xe3J\xef^\xa8:\x10a\x17r\xc3\x18\x98\xc3\xc5\x8d\x9c\x1d\xc12;\x97^A\xa6\xb3\x83\xb2\x1fbgg\xc2}\xa9\xdf!\x00\xae\xb7\x8fWb\xe5\xeb\xdc\x0c]C\xfd\xde&|\xd2\x85)-\x80\xba\xde\xfd\xe3\x15,x\x9b\x03\xa7\x167\xb4\xf2<\xc6\xcb\xca6\xca\x81\xdb\x96\xaf\xef>\xb9\x9b\x14\xf9$\xbd\x15\xea\xa9\xe3\x04@\x17\xe6\x08\x82\'>\x80\xe5\xec\x9d@\xe6\xf8\xa5\xf1\x0f\x80\xddI\x11\\\xe6*\xf2\xf4ZxT\xff\x0e,&p7\xb0\xd1\xa0\xf42\xc2\xa4\x073t\xad\x9bGf\xcd\xc1a\xa1\x8b%\xb1\x15\xa7,ld\xc0c4\x1fM\n\xff\x98\x9c\x99\xc3U\xf0,Jy\xf37\x0fFO\xe7\xf1\x93y>l\xb3>\x83O\xe8\xd1\xdd\x9b5KS\x12\xde2L\x12\x18\xa6\xe3\x04\xc3\x9c\x8e\x8e$F\x894\xf6\xbf\xf1?\xa7I\xa0\xd0w\x823S_C2#0L2(\xf4\x1da\x98\xfa\x1a\x92\x19\x81a\x92A\xa1\xef\x08\xc3\xd4\xd7\x90\xcc\x08\x0c\x93\x0c\n}G\x18\xa6\xbe\x86dF`\x98dP\xe8;\xc20\xf55$3\x02\xc3$\x83B\xdf\x91\xdf\xffr\xec\xc6\xdb\xb0\x1e\xeb\x00\x00\x00\x00IEND\xaeB`\x82',
        ),
        ],
    from_values=EmailAddress(name='sh', email='samgentlemankaka@foxmail.com'),
    to_values=(EmailAddress(name='qmebi', email='qmebi@mfk.app'),),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(),
)


import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='undef method parameter bug',
    from_='Big Bug bb@bug.com',
    to=('rubymail@ruby-lang.org',),
    cc=(),
    bcc=(),
    reply_to=(),
    date=datetime.datetime(2010, 2, 19, 10, 8, 29, tzinfo=datetime.timezone(datetime.timedelta(seconds=10800))),
    date_str='Fri, 19 Feb 2010 10:08:29 +0300',
    text='foo bar\r\n',
    html='',
    headers={'from': ('Big Bug bb@bug.com',), 'to': ('rubymail@ruby-lang.org',), 'subject': ('undef method parameter bug',), 'date': ('Fri, 19 Feb 2010 10:08:29 +0300',), 'mime-version': ('1.0',), 'content-type': ('Text/Plain; charset="iso-8859-1"',), 'content-transfer-encoding': ('quoted-printable',), 'message-id': ('201002191008.30117.foo.bar@company.com',)},
    attachments=[],
    from_values=EmailAddress(name='', email='Big Bug bb@bug.com'),
    to_values=(EmailAddress(name='', email='rubymail@ruby-lang.org'),),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(),
)


import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='Warning: could not send message for past 1 day',
    from_='MAILER-DAEMON@antivirus.uqam.ca',
    to=('roor32@gmail.com',),
    cc=(),
    bcc=(),
    reply_to=(),
    date=datetime.datetime(2006, 10, 20, 4, 28, 33, tzinfo=datetime.timezone(datetime.timedelta(days=-1, seconds=72000))),
    date_str='Fri, 20 Oct 2006 04:28:33 -0400 (EDT)',
    text="    **********************************************\r\n    **      THIS IS A WARNING MESSAGE ONLY      **\r\n    **  YOU DO NOT NEED TO RESEND YOUR MESSAGE  **\r\n    **********************************************\r\n\r\nThe original message was received at Thu, 19 Oct 2006 01:23:47 -0400 (EDT)\r\nfrom py-out-1112.google.com [64.233.166.178]\r\n\r\n   ----- Transcript of session follows -----\r\n<larose.julie@courrier.uqam.ca>... Deferred\r\nWarning: message still undelivered after 1 day\r\nWill keep trying until message is 5 days old\r\nSalut,\r\nje ne suis pas sur de m'adresser a la bonne personne.\r\nMais si jamais tu me reconnais :p, peux-tu repondre a ce mail ?\r\nMerci.\r\nRoger\r\n",
    html='',
    headers={'x-gmail-received': ('b2c98353cc39d93e4204831226f778c200d28109',), 'delivered-to': ('roor32@gmail.com',), 'received': ('by 10.64.10.4 with SMTP id 4cs594808qbj;\r\n        Fri, 20 Oct 2006 01:53:34 -0700 (PDT)', 'by 10.65.224.11 with SMTP id b11mr149813qbr;\r\n        Fri, 20 Oct 2006 01:53:34 -0700 (PDT)', 'from anis.telecom.uqam.ca (anis.telecom.uqam.ca [132.208.250.6])\r\n        by mx.google.com with ESMTP id e13si1575402qbe.2006.10.20.01.53.34;\r\n        Fri, 20 Oct 2006 01:53:34 -0700 (PDT)', 'from anis4.telecom.uqam.ca (anis4.telecom.uqam.ca [132.208.250.236])\r\n\tby sortant.uqam.ca (8.13.6/8.12.1) with SMTP id k9K8SIwA023763\r\n\tfor <roor32@gmail.com>; Fri, 20 Oct 2006 04:28:33 -0400 (EDT)', 'from antivirus.uqam.ca ([127.0.0.1])\r\n by anis4.telecom.uqam.ca (SAVSMTP 3.1.1.32) with SMTP id M2006102004283404041\r\n for <roor32@gmail.com>; Fri, 20 Oct 2006 04:28:34 -0400', 'from localhost (localhost)\r\n\tby antivirus.uqam.ca (8.13.6/8.12.1) id k9JMDbg4005560;\r\n\tFri, 20 Oct 2006 04:28:33 -0400 (EDT)'), 'return-path': ('<>',), 'received-spf': ('pass (google.com: best guess record for domain of anis.telecom.uqam.ca designates 132.208.250.6 as permitted sender)',), 'date': ('Fri, 20 Oct 2006 04:28:33 -0400 (EDT)',), 'from': ('Mail Delivery Subsystem <MAILER-DAEMON@antivirus.uqam.ca>',), 'message-id': ('<200610200828.k9JMDbg4005560@antivirus.uqam.ca>',), 'to': ('<roor32@gmail.com>',), 'mime-version': ('1.0',), 'content-type': ('multipart/report; report-type=delivery-status;\r\n\tboundary="k9JMDbg4005560.1161332913/antivirus.uqam.ca"',), 'subject': ('Warning: could not send message for past 1 day',), 'auto-submitted': ('auto-generated (warning-timeout)',)},
    attachments=[
        dict(
            filename='',
            content_id='',
            content_disposition='',
            content_type='message/rfc822',
            payload=b'Return-Path: <roor32@gmail.com>\nReceived: from py-out-1112.google.com (py-out-1112.google.com\n [64.233.166.178])\n\tby intrant.uqam.ca (8.13.6/8.12.2/uqam-filtres) with SMTP id k9J5Ni8o007263\n\tfor <larose.julie@courrier.uqam.ca>; Thu, 19 Oct 2006 01:23:47 -0400 (EDT)\nX-UQAM-Spam-Filter: Filtre-Uqam re:  abuse@uqam.ca\nReceived: by py-out-1112.google.com with SMTP id t32so610221pyc\n        for <larose.julie@courrier.uqam.ca>;\n Wed, 18 Oct 2006 22:23:33 -0700 (PDT)\nDomainKey-Signature: a=rsa-sha1; q=dns; c=nofws;\n        s=beta; d=gmail.com;\n        h=received:message-id:date:from:to:subject:mime-version:content-type:content-transfer-encoding:content-disposition;\n        b=Fwt0k1kaMU+1kM1iTG0q4Xf94p9alSohgM7QQN0CSjfBYUUhT+J4Y6+ZWotaaSffV3gX86RE/n97n0yQ/33EgYKIifuEpa0hi2mg3KTmcDqlCjiDfih58Z998GEFfbhu0he2jsoB+k6AgVRFPwP6LMRi6T66vr2f7IOAmX2IHiU=\nReceived: by 10.65.241.20 with SMTP id t20mr15391984qbr;\n        Wed, 18 Oct 2006 15:10:03 -0700 (PDT)\nReceived: by 10.64.10.4 with HTTP; Wed, 18 Oct 2006 15:10:03 -0700 (PDT)\nMessage-ID: <89d7557c0610181510r6fa5ebd8n66a7deec71ade118@mail.gmail.com>\nDate: Wed, 18 Oct 2006 18:10:03 -0400\nFrom: "=?ISO-8859-1?Q?RogE9?=" <roor32@gmail.com>\nTo: larose.julie@courrier.uqam.ca\nSubject: Est-ce toi ?\nMIME-Version: 1.0\nContent-Type: text/plain; charset=ISO-8859-1; format=flowed\nContent-Transfer-Encoding: 7bit\nContent-Disposition: inline\n\nSalut,\nje ne suis pas sur de m\'adresser a la bonne personne.\nMais si jamais tu me reconnais :p, peux-tu repondre a ce mail ?\nMerci.\nRoger\n',
        ),
        ],
    from_values=EmailAddress(name='Mail Delivery Subsystem', email='MAILER-DAEMON@antivirus.uqam.ca'),
    to_values=(EmailAddress(name='', email='roor32@gmail.com'),),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(),
)


import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='testing',
    from_='foo@example.com',
    to=('blah@example.com',),
    cc=(),
    bcc=(),
    reply_to=(),
    date=datetime.datetime(2005, 6, 6, 22, 21, 22, tzinfo=datetime.timezone(datetime.timedelta(seconds=7200))),
    date_str='Mon, 6 Jun 2005 22:21:22 +0200',
    text='This is the first part.\r\n',
    html='',
    headers={'mime-version': ('1.0 (Apple Message framework v730)',), 'content-type': ('multipart/mixed; boundary=Apple-Mail-13-196941151',), 'message-id': ('<9169D984-4E0B-45EF-82D4-8F5E53AD7012@example.com>',), 'from': ('foo@example.com',), 'subject': ('testing',), 'date': ('Mon, 6 Jun 2005 22:21:22 +0200',), 'to': ('blah@example.com',)},
    attachments=[
        dict(
            filename='This is a test.txt',
            content_id='',
            content_disposition='attachment',
            content_type='text/plain',
            payload=b'Hi there.\r\n',
        ),
        ],
    from_values=EmailAddress(name='', email='foo@example.com'),
    to_values=(EmailAddress(name='', email='blah@example.com'),),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(),
)


import datetime
from imap_tools import EmailAddress

DATA = dict(
    subject='illegal copy of our patient education software ',
    from_='ak@g.com',
    to=('abuser@r.ru',),
    cc=(),
    bcc=(),
    reply_to=('ak@g.com',),
    date=datetime.datetime(2009, 9, 19, 19, 49, 36, tzinfo=datetime.timezone(datetime.timedelta(seconds=14400))),
    date_str='Sat, 19 Sep 2009 19:49:36 +0400',
    text='',
    html='',
    headers={'return-path': ('<ak@g.com>', '<ak@g.com>'), 'envelope-to': ('abuser@r.ru',), 'delivery-date': ('Sat, 19 Sep 2009 21:41:40 +0400',), 'received': ('from mail-fx0-f215.google.com ([209.85.220.215]:39047)\r\n\tby mail.rg.com with esmtp \r\n\tid 1Mp3wA-0007kA-Ic\r\n\tfor <abuser@r.ru>; Sat, 19 Sep 2009 21:41:30 +0400', 'by fxm11 with SMTP id 11so1412272fxm.15\r\n        for <abuser@r.ru>; Sat, 19 Sep 2009 10:40:39 -0700 (PDT)', 'by 10.86.240.9 with SMTP id n9mr2799028fgh.70.1253375382709;\r\n        Sat, 19 Sep 2009 08:49:42 -0700 (PDT)', 'from articondell (ppp85-140-104-88.pppoe.mtu-net.ru [85.140.104.88])\r\n        by mx.google.com with ESMTPS id e11sm2485072fga.21.2009.09.19.08.49.39\r\n        (version=SSLv3 cipher=RC4-MD5);\r\n        Sat, 19 Sep 2009 08:49:40 -0700 (PDT)'), 'dkim-signature': ('v=1; a=rsa-sha256; c=relaxed/relaxed;\r\n        d=gmail.com; s=gamma;\r\n        h=domainkey-signature:received:received:from:to:subject:date\r\n         :message-id:mime-version:content-type:x-priority:x-msmail-priority\r\n         :x-mailer:importance:x-mimeole:in-reply-to;\r\n        bh=U4LmZ2XrYxpE2gzziKYLSJXvmTsl0JSdrp4OYcIw2xw=;\r\n        b=N4ZFNsnhqgG0qILwgjv0Sh0qKgSR+A5hMn60yxtSlACUDq/xQ/52pJkCuMChX0Pzxo\r\n         HaRYdiAsxyzlzmSStwtM/fAHZSNrXD0pLpeQOCi1r8ZSyQ6mKb4WgO56FFNOjCA0rLl8\r\n         NOelymNmCIYTwuYz5Dd0PthnWL0YZPU+YUEHU=',), 'domainkey-signature': ('a=rsa-sha1; c=nofws;\r\n        d=gmail.com; s=gamma;\r\n        h=from:to:subject:date:message-id:mime-version:content-type\r\n         :x-priority:x-msmail-priority:x-mailer:importance:x-mimeole\r\n         :in-reply-to;\r\n        b=ZTs05s6OpCmZp0GWYqGIv8xGM+AmM7+QL6SdaR45KgwCcN8sJH0SB4PI4QMFSW3+ZX\r\n         IdYVUGjqer/bEz7POPS8FOTCZW1QTkRcTHqVEFezAmotlc0VfLVSnnW9oRpzJ/UFQUWM\r\n         +Wn0vcObiGmrUPUJWWiNgBvuRnrJHk+nICCQI=',), 'from': ('"Andrey Kuznetsov" <ak@g.com>',), 'to': ('<abuser@r.ru>',), 'subject': ('illegal copy of our patient education software ',), 'date': ('Sat, 19 Sep 2009 19:49:36 +0400',), 'message-id': ('<F194F88AF3E341A6B2B135CC17912811@articondell>',), 'mime-version': ('1.0',), 'content-type': ('multipart/related;\r\n\tboundary="----=_NextPart_000_000A_01CA3962.51E52370"',), 'x-priority': ('3 (Normal)',), 'x-msmail-priority': ('Normal',), 'x-mailer': ('Microsoft Outlook, Build 10.0.4024',), 'importance': ('Normal',), 'x-mimeole': ('Produced By Microsoft MimeOLE V6.00.2900.5579',), 'in-reply-to': ('',), 'recivied-spf': ('(invalid) ',), 'rspam-score': ('40',), 'list-id': ('abuser@rg.com Mailing List',), 'reply-to': ('ak@g.com',)},
    attachments=[],
    from_values=EmailAddress(name='Andrey Kuznetsov', email='ak@g.com'),
    to_values=(EmailAddress(name='', email='abuser@r.ru'),),
    cc_values=(),
    bcc_values=(),
    reply_to_values=(EmailAddress(name='', email='ak@g.com'),),
)